#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Demonstrate usage of LFPy.NetworkCell with a ball-and-stick type
morphology with active HH channels inserted in the soma and passive-leak
channels distributed throughout the apical stick. The corresponding morphology
and template-specification is in the files BallAndStick.hoc and
BallAndStickTemplate.hoc.

Execution:

    python example_network_cell.py

Copyright (C) 2017 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""
# import modules:
from matplotlib.pyplot import subplot, plot, ylabel, xlabel, show, close
import neuron
from LFPy import NetworkCell, StimIntElectrode
# class NetworkCell parameters:
cellParameters = dict(
    morphology='BallAndStick.hoc',
    templatefile='BallAndStickTemplate.hoc',
    templatename='BallAndStickTemplate',
    templateargs=None,
    v_init=-65.
)
# create cell:
cell = NetworkCell(
    tstart=0., tstop=100.,
    **cellParameters
)
# create stimulus device:
iclamp = StimIntElectrode(
    cell=cell,
    idx=0,
    pptype='IClamp',
    amp=0.5,
    dur=80.,
    delay=10.,
    record_current=True
)
# run simulation:
cell.simulate()

# plot cell response:
subplot(2, 1, 1)
plot(cell.tvec, iclamp.i)
ylabel(r'$i_\mathrm{clamp}$ (nA)')
subplot(2, 1, 2)
plot(cell.tvec, cell.somav)
ylabel(r'$V_\mathrm{soma}$ (mV)')
xlabel(r'$t$ (ms)')
show()

# customary cleanup of object references
# allowing for consecutive runs:
close('all')
iclamp = None
cell = None
neuron.h('forall delete_section()')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Copyright (C) 2012 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""


import numpy as np


def return_dist_from_segments(xstart, ystart, zstart, xend, yend, zend, p):
    """
    Returns distance and closest point on line segments from point p
    """
    px = xend-xstart
    py = yend-ystart
    pz = zend-zstart

    delta = px*px + py*py + pz*pz
    u = ((p[0] - xstart) * px + (p[1] - ystart) * py + (p[2] - zstart) * pz) / delta
    u[u > 1] = 1.0
    u[u < 0] = 0.0

    closest_point = np.array([xstart + u * px,
                              ystart + u * py,
                              zstart + u * pz])
    dist = np.sqrt(np.sum((closest_point.T - p)**2, axis=1))
    return dist, closest_point


def calc_lfp_linesource_anisotropic(cell, x, y, z, sigma, r_limit):
    """Calculate electric field potential using the line-source method, all
    compartments treated as line sources, even soma.

    Parameters
    ----------
    cell: obj
        LFPy.Cell or LFPy.TemplateCell instance
    x : float
        extracellular position, x-axis
    y : float
        extracellular position, y-axis
    z : float
        extracellular position, z-axis
    sigma : array
        extracellular conductivity [sigma_x, sigma_y, sigma_z]
    r_limit : np.ndarray
        minimum distance to source current for each compartment
    """

    #some variables for h, r2, r_soma calculations
    xstart = cell.xstart
    xend = cell.xend
    ystart = cell.ystart
    yend = cell.yend
    zstart = cell.zstart
    zend = cell.zend
    l_vecs = np.array([xend - xstart,
                      yend - ystart,
                      zend - zstart])

    pos = np.array([x, y, z])

    rs, closest_points = return_dist_from_segments(xstart, ystart, zstart,
                                                   xend, yend, zend, pos)

    dx2 = (xend - xstart)**2
    dy2 = (yend - ystart)**2
    dz2 = (zend - zstart)**2
    a = (sigma[1] * sigma[2] * dx2 +
         sigma[0] * sigma[2] * dy2 +
         sigma[0] * sigma[1] * dz2)

    b = -2 * (sigma[1] * sigma[2] * (x - xstart) * (xend - xstart) +
              sigma[0] * sigma[2] * (y - ystart) * (yend - ystart) +
              sigma[0] * sigma[1] * (z - zstart) * (zend - zstart))
    c = (sigma[1] * sigma[2] * (x - xstart)**2 +
         sigma[0] * sigma[2] * (y - ystart)**2 +
         sigma[0] * sigma[1] * (z - zstart)**2)

    for idx in np.where(rs < r_limit)[0]:
        r, closest_point, l_vec = rs[idx], closest_points[:, idx], l_vecs[:, idx]

        p_ = pos.copy()
        if np.abs(r) < 1e-12:
            if np.abs(l_vec[0]) < 1e-12:
                p_[0] += r_limit[idx]
            elif np.abs(l_vec[1]) < 1e-12:
                p_[1] += r_limit[idx]
            elif np.abs(l_vec[2]) < 1e-12:
                p_[2] += r_limit[idx]
            else:
                displace_vec = np.array([-l_vec[1], l_vec[0], 0])
                displace_vec = displace_vec / np.sqrt(np.sum(displace_vec**2)) * r_limit[idx]
                p_[:] += displace_vec
        else:
            p_[:] = pos + (pos - closest_point) * (r_limit[idx] - r) / r

        if np.sqrt(np.sum((p_ - closest_point)**2)) - r_limit[idx] > 1e-9:
            print(p_, closest_point)

            raise RuntimeError("Segment adjustment not working")

        b[idx] = -2 * (sigma[1] * sigma[2] * (p_[0] - xstart[idx]) * (xend[idx] - xstart[idx]) +
                   sigma[0] * sigma[2] * (p_[1] - ystart[idx]) * (yend[idx] - ystart[idx]) +
                   sigma[0] * sigma[1] * (p_[2] - zstart[idx]) * (zend[idx] - zstart[idx]))
        c[idx] = (sigma[1] * sigma[2] * (p_[0] - xstart[idx])**2 +
              sigma[0] * sigma[2] * (p_[1] - ystart[idx])**2 +
              sigma[0] * sigma[1] * (p_[2] - zstart[idx])**2)

    [i] = np.where(np.abs(b) <= 1e-6)
    [iia] = np.where(np.bitwise_and(np.abs(4 * a * c - b*b) < 1e-6, np.abs(a - c) < 1e-6))
    [iib] = np.where(np.bitwise_and(np.abs(4 * a * c - b*b) < 1e-6, np.abs(a - c) >= 1e-6))
    [iii] = np.where(np.bitwise_and(4 * a * c - b*b < -1e-6, np.abs(b) > 1e-6))
    [iiii] = np.where(np.bitwise_and(4 * a * c - b*b > 1e-6, np.abs(b) > 1e-6))

    if len(i) + len(iia) + len(iib) + len(iii) + len(iiii) != cell.totnsegs:
        print(a, b, c)
        print(i, iia, iib, iii, iiii)
        raise RuntimeError

    mapping = np.zeros(cell.totnsegs)
    mapping[i] = _anisotropic_line_source_case_i(a[i], c[i])
    mapping[iia] = _anisotropic_line_source_case_iia(a[iia], c[iia])
    mapping[iib] = _anisotropic_line_source_case_iib(a[iib], b[iib], c[iib])
    mapping[iii] = _anisotropic_line_source_case_iii(a[iii], b[iii], c[iii])
    mapping[iiii] = _anisotropic_line_source_case_iiii(a[iiii], b[iiii], c[iiii])

    if np.isnan(mapping).any():
        raise RuntimeError("NaN")

    return 1 / (4 * np.pi) * mapping / np.sqrt(a)


def calc_lfp_soma_as_point_anisotropic(cell, x, y, z, sigma, r_limit):
    """Calculate electric field potential, soma is treated as point source, all
    compartments except soma are treated as line sources.

    Parameters
    ----------
    cell: obj
        LFPy.Cell or LFPy.TemplateCell instance
    x : float
        extracellular position, x-axis
    y : float
        extracellular position, y-axis
    z : float
        extracellular position, z-axis
    sigma : array
        extracellular conductivity [sigma_x, sigma_y, sigma_z]
    r_limit : np.ndarray
        minimum distance to source current for each compartment
    """

    xstart = cell.xstart
    xend = cell.xend
    ystart = cell.ystart
    yend = cell.yend
    zstart = cell.zstart
    zend = cell.zend
    l_vecs = np.array([xend - xstart, yend - ystart, zend - zstart])

    pos = np.array([x, y, z])

    rs, closest_points = return_dist_from_segments(xstart, ystart, zstart, xend, yend, zend, pos)

    dx2 = (xend - xstart)**2
    dy2 = (yend - ystart)**2
    dz2 = (zend - zstart)**2
    a = (sigma[1] * sigma[2] * dx2 +
         sigma[0] * sigma[2] * dy2 +
         sigma[0] * sigma[1] * dz2)

    b = -2 * (sigma[1] * sigma[2] * (x - xstart) * (xend - xstart) +
              sigma[0] * sigma[2] * (y - ystart) * (yend - ystart) +
              sigma[0] * sigma[1] * (z - zstart) * (zend - zstart))
    c = (sigma[1] * sigma[2] * (x - xstart)**2 +
         sigma[0] * sigma[2] * (y - ystart)**2 +
         sigma[0] * sigma[1] * (z - zstart)**2)

    for idx in np.where(rs < r_limit)[0]:
        r, closest_point, l_vec = rs[idx], closest_points[:, idx], l_vecs[:, idx]

        p_ = pos.copy()
        if np.abs(r) < 1e-12:
            if np.abs(l_vec[0]) < 1e-12:
                p_[0] += r_limit[idx]
            elif np.abs(l_vec[1]) < 1e-12:
                p_[1] += r_limit[idx]
            elif np.abs(l_vec[2]) < 1e-12:
                p_[2] += r_limit[idx]
            else:
                displace_vec = np.array([-l_vec[1], l_vec[0], 0])
                displace_vec = displace_vec / np.sqrt(np.sum(displace_vec**2)) * r_limit[idx]
                p_[:] += displace_vec
        else:
            p_[:] = pos + (pos - closest_point) * (r_limit[idx] - r) / r

        if np.sqrt(np.sum((p_ - closest_point)**2)) - r_limit[idx] > 1e-9:
            print(p_, closest_point)

            raise RuntimeError("Segment adjustment not working")

        b[idx] = -2 * (sigma[1] * sigma[2] * (p_[0] - xstart[idx]) * (xend[idx] - xstart[idx]) +
                   sigma[0] * sigma[2] * (p_[1] - ystart[idx]) * (yend[idx] - ystart[idx]) +
                   sigma[0] * sigma[1] * (p_[2] - zstart[idx]) * (zend[idx] - zstart[idx]))
        c[idx] = (sigma[1] * sigma[2] * (p_[0] - xstart[idx])**2 +
              sigma[0] * sigma[2] * (p_[1] - ystart[idx])**2 +
              sigma[0] * sigma[1] * (p_[2] - zstart[idx])**2)

    [i] = np.where(np.abs(b) <= 1e-6)
    [iia] = np.where(np.bitwise_and(np.abs(4 * a * c - b*b) < 1e-6, np.abs(a - c) < 1e-6))
    [iib] = np.where(np.bitwise_and(np.abs(4 * a * c - b*b) < 1e-6, np.abs(a - c) >= 1e-6))
    [iii] = np.where(np.bitwise_and(4 * a * c - b*b < -1e-6, np.abs(b) > 1e-6))
    [iiii] = np.where(np.bitwise_and(4 * a * c - b*b > 1e-6, np.abs(b) > 1e-6))

    if len(i) + len(iia) + len(iib) + len(iii) + len(iiii) != cell.totnsegs:
        print(a, b, c)
        print(i, iia, iib, iii, iiii)
        raise RuntimeError

    mapping = np.zeros(cell.totnsegs)
    mapping[i] = _anisotropic_line_source_case_i(a[i], c[i])
    mapping[iia] = _anisotropic_line_source_case_iia(a[iia], c[iia])
    mapping[iib] = _anisotropic_line_source_case_iib(a[iib], b[iib], c[iib])
    mapping[iii] = _anisotropic_line_source_case_iii(a[iii], b[iii], c[iii])
    mapping[iiii] = _anisotropic_line_source_case_iiii(a[iiii], b[iiii], c[iiii])

    if np.isnan(mapping).any():
        raise RuntimeError("NaN")

    mapping /= np.sqrt(a)

    # Get compartment indices for somatic compartments (to be treated as point
    # sources)
    try:
        somainds = cell.get_idx("soma")
    except Exception:
        raise Exception('Call {}.get_idx("soma") failed in method LFPy.lfpcalc.calc_lfp_soma_as_point'.format(cell))
    
    dx2_soma = (cell.xmid[somainds] - x)**2
    dy2_soma = (cell.ymid[somainds] - y)**2
    dz2_soma = (cell.zmid[somainds] - z)**2

    r2_soma = dx2_soma + dy2_soma + dz2_soma

    # Go through and correct all (if any) somatic idxs that are too close
    for close_idx in np.where(np.abs(r2_soma) < 1e-6)[0]:
        dx2_soma[close_idx] += 0.001
        r2_soma[close_idx] += 0.001

    for close_idx in np.where(r2_soma < r_limit[somainds]**2)[0]:
        # For anisotropic media, the direction in which to move points matter.
        # Radial distance between point source and electrode is scaled to r_limit
        r2_scale_factor = r_limit[somainds[close_idx]]*r_limit[somainds[close_idx]] / r2_soma[close_idx]
        dx2_soma[close_idx] *= r2_scale_factor
        dy2_soma[close_idx] *= r2_scale_factor
        dz2_soma[close_idx] *= r2_scale_factor

    mapping[somainds] = 1/np.sqrt(sigma[1] * sigma[2] * dx2_soma
                    + sigma[0] * sigma[2] * dy2_soma
                    + sigma[0] * sigma[1] * dz2_soma)

    return 1. / (4 * np.pi) * mapping


def _anisotropic_line_source_case_i(a, c):
    return np.log(np.sqrt(a / c) + np.sqrt(a / c + 1))


def _anisotropic_line_source_case_iia(a, c):
    return np.log(np.abs(1 + np.sqrt(a / c)))


def _anisotropic_line_source_case_iib(a, b, c):
    return np.abs(np.log(np.abs(np.sign(b) * np.sqrt(a/c) + 1)))


def _anisotropic_line_source_case_iii(a, b, c):
    return np.log(np.abs((2 * a + b + 2 * np.sqrt(a * (a + b + c)))
                        / (b + 2 * np.sqrt(a * c))))


def _anisotropic_line_source_case_iiii(a, b, c):
    return (np.arcsinh((2 * a + b) / np.sqrt(4 * a * c - b*b)) -
                        np.arcsinh(b / np.sqrt(4 * a * c - b*b)))


def calc_lfp_linesource(cell, x, y, z, sigma, r_limit):

    """Calculate electric field potential using the line-source method, all
    compartments treated as line sources, including soma.
    
    Parameters
    ----------        
    cell: obj
        LFPy.Cell or LFPy.TemplateCell like instance
    x : float
        extracellular position, x-axis
    y : float
        extracellular position, y-axis
    z : float
        extracellular position, z-axis
    sigma : float
        extracellular conductivity
    r_limit : np.ndarray
        minimum distance to source current for each compartment
    """

    #some variables for h, r2, r_soma calculations
    xstart = cell.xstart
    xend = cell.xend
    ystart = cell.ystart
    yend = cell.yend
    zstart = cell.zstart
    zend = cell.zend
    
    deltaS = _deltaS_calc(xstart, xend, ystart, yend, zstart, zend)
    h = _h_calc(xstart, xend, ystart, yend, zstart, zend, deltaS, x, y, z)
    r2 = _r2_calc(xend, yend, zend, x, y, z, h)

    too_close_idxs = np.where(r2 < r_limit*r_limit)[0]
    r2[too_close_idxs] = r_limit[too_close_idxs]**2
    l = h + deltaS
    hnegi = h < 0
    hposi = h >= 0
    lnegi = l < 0
    lposi = l >= 0

    mapping = np.zeros(len(cell.xstart))

    #case i, h < 0, l < 0
    [i] = np.where(hnegi & lnegi)
    #case ii, h < 0, l >= 0
    [ii] = np.where(hnegi & lposi)
    #case iii, h >= 0, l >= 0
    [iii] = np.where(hposi & lposi)


    mapping[i] = _linesource_calc_case1(l[i], r2[i], h[i])
    mapping[ii] = _linesource_calc_case2(l[ii], r2[ii], h[ii])
    mapping[iii] = _linesource_calc_case3(l[iii], r2[iii], h[iii])
    return 1 / (4 * np.pi * sigma * deltaS) * mapping


def calc_lfp_soma_as_point(cell, x, y, z, sigma, r_limit):

    """Calculate electric field potential using the line-source method,
    soma is treated as point/sphere source
    
    Parameters
    ----------
    cell: obj
        `LFPy.Cell` or `LFPy.TemplateCell` like instance
    x : float
        extracellular position, x-axis
    y : float
        extracellular position, y-axis
    z : float
        extracellular position, z-axis
    sigma : float
        extracellular conductivity in S/m
    r_limit : np.ndarray
        minimum distance to source current for each compartment.
    """
    # get compartment indices for somatic compartments (to be treated as point
    # sources)
    try:
        somainds = cell.get_idx("soma")
    except Exception:
        raise Exception('Call {}.get_idx("soma") failed in method LFPy.lfpcalc.calc_lfp_soma_as_point'.format(cell))

    #some variables for h, r2, r_soma calculations
    xstart = cell.xstart
    xmid = cell.xmid[somainds]
    xend = cell.xend
    ystart = cell.ystart
    ymid = cell.ymid[somainds]
    yend = cell.yend
    zstart = cell.zstart
    zmid = cell.zmid[somainds]
    zend = cell.zend

    deltaS = _deltaS_calc(xstart, xend, ystart, yend, zstart, zend)
    h = _h_calc(xstart, xend, ystart, yend, zstart, zend, deltaS, x, y, z)
    r2 = _r2_calc(xend, yend, zend, x, y, z, h)
    r_soma = _r_soma_calc(xmid, ymid, zmid, x, y, z)
    if np.any(r_soma < r_limit[somainds]):
        print('Adjusting r-distance to soma segments')
        r_soma[r_soma < r_limit[somainds]
               ] = r_limit[somainds][r_soma < r_limit[somainds]]

    too_close_idxs = np.where(r2 < r_limit*r_limit)[0]
    r2[too_close_idxs] = r_limit[too_close_idxs]**2
    l = h + deltaS

    hnegi = h < 0
    hposi = h >= 0
    lnegi = l < 0
    lposi = l >= 0

    # Ensuring that soma is not treated as line-source
    hnegi[somainds] = hposi[somainds] = lnegi[somainds] = lposi[somainds] = False

    #Line sources
    #case i,  h < 0,  l < 0
    i = np.where(hnegi & lnegi)
    #case ii,  h < 0,  l >= 0
    ii = np.where(hnegi & lposi)
    #case iii,  h >= 0,  l >= 0
    iii = np.where(hposi & lposi)

    #Summarizing all potential contributions
    mapping = np.zeros(cell.totnsegs)
    mapping[somainds] = 1 / r_soma
    deltaS[somainds] = 1.

    mapping[i] = _linesource_calc_case1(l[i], r2[i], h[i])
    mapping[ii] = _linesource_calc_case2(l[ii], r2[ii], h[ii])
    mapping[iii] = _linesource_calc_case3(l[iii], r2[iii], h[iii])

    return 1 / (4 * np.pi * sigma * deltaS) * mapping


def _linesource_calc_case1(l_i, r2_i, h_i):
    """Calculates linesource contribution for case i"""
    bb = np.sqrt(h_i*h_i + r2_i) - h_i
    cc = np.sqrt(l_i*l_i + r2_i) - l_i
    dd = np.log(bb / cc)
    return dd


def _linesource_calc_case2(l_ii, r2_ii, h_ii):
    """Calculates linesource contribution for case ii"""
    bb = np.sqrt(h_ii*h_ii + r2_ii) - h_ii
    cc = (l_ii + np.sqrt(l_ii*l_ii + r2_ii)) / r2_ii
    dd = np.log(bb * cc)
    return dd


def _linesource_calc_case3(l_iii, r2_iii, h_iii):
    """Calculates linesource contribution for case iii"""
    bb = np.sqrt(l_iii*l_iii + r2_iii) + l_iii
    cc = np.sqrt(h_iii*h_iii + r2_iii) + h_iii
    dd = np.log(bb / cc)
    return dd


def _deltaS_calc(xstart, xend, ystart, yend, zstart, zend):
    """Returns length of each segment"""
    deltaS = np.sqrt((xstart - xend)**2 + (ystart - yend)**2 +
                     (zstart-zend)**2)
    return deltaS


def _h_calc(xstart, xend, ystart, yend, zstart, zend, deltaS, x, y, z):
    """Subroutine used by calc_lfp_*()"""
    aa = np.array([x - xend, y - yend, z-zend])
    bb = np.array([xend - xstart, yend - ystart, zend - zstart])
    cc = np.sum(aa*bb, axis=0)
    hh = cc / deltaS
    return hh


def _r2_calc(xend, yend, zend, x, y, z, h):
    """Subroutine used by calc_lfp_*()"""
    r2 = (x-xend)**2 + (y-yend)**2 + (z-zend)**2 - h**2
    return abs(r2)


def _r_soma_calc(xmid, ymid, zmid, x, y, z):
    """calculate the distance to soma midpoint"""
    r_soma = np.sqrt((x - xmid)**2 + (y - ymid)**2 + (z - zmid)**2)
    return r_soma


def calc_lfp_pointsource(cell, x, y, z, sigma, r_limit):
    """Calculate extracellular potentials using the point-source
    equation on all compartments

    Parameters
    ----------
    cell: obj
        LFPy.Cell or LFPy.TemplateCell like instance
    x : float
        extracellular position, x-axis
    y : float
        extracellular position, y-axis
    z : float
        extracellular position, z-axis
    sigma : float
        extracellular conductivity
    r_limit : np.ndarray
        minimum distance to source current for each compartment
    """

    r2 = (cell.xmid - x)**2 + (cell.ymid - y)**2 + (cell.zmid - z)**2
    r2 = _check_rlimit_point(r2, r_limit)
    mapping = 1 / (4 * np.pi * sigma * np.sqrt(r2))
    return mapping


def calc_lfp_pointsource_anisotropic(cell, x, y, z, sigma, r_limit):
    """Calculate extracellular potentials using the anisotropic point-source
    equation on all compartments

    Parameters
    ----------
    cell: obj
        LFPy.Cell or LFPy.TemplateCell instance
    x : float
        extracellular position, x-axis
    y : float
        extracellular position, y-axis
    z : float
        extracellular position, z-axis
    sigma : array
        extracellular conductivity in [x,y,z]-direction
    r_limit : np.ndarray
        minimum distance to source current for each compartment
    """

    dx2 = (cell.xmid - x)**2
    dy2 = (cell.ymid - y)**2
    dz2 = (cell.zmid - z)**2

    r2 = dx2 + dy2 + dz2
    if (np.abs(r2) < 1e-6).any():
        dx2[np.abs(r2) < 1e-6] += 0.001
        r2[np.abs(r2) < 1e-6] += 0.001

    close_idxs = r2 < r_limit*r_limit

    # For anisotropic media, the direction in which to move points matter.
    # Radial distance between point source and electrode is scaled to r_limit
    r2_scale_factor = r_limit[close_idxs]*r_limit[close_idxs] / r2[close_idxs]
    dx2[close_idxs] *= r2_scale_factor
    dy2[close_idxs] *= r2_scale_factor
    dz2[close_idxs] *= r2_scale_factor

    sigma_r = np.sqrt(sigma[1] * sigma[2] * dx2
                    + sigma[0] * sigma[2] * dy2
                    + sigma[0] * sigma[1] * dz2)

    mapping = 1 / (4 * np.pi * sigma_r)
    return mapping


def _check_rlimit_point(r2, r_limit):
    """Correct r2 so that r2 >= r_limit**2 for all values"""
    inds = r2 < r_limit*r_limit
    r2[inds] = r_limit[inds]*r_limit[inds]
    return r2


def calc_lfp_pointsource_moi(cell, x, y, z, sigma_T, sigma_S, sigma_G,
                             steps, h, r_limit, **kwargs):
    """Calculate extracellular potentials using the point-source
    equation on all compartments for in vitro Microelectrode Array (MEA) slices

    Parameters
    ----------
    cell: obj
        LFPy.Cell or LFPy.TemplateCell like instance
    x : float
        extracellular position, x-axis
    y : float
        extracellular position, y-axis
    z : float
        extracellular position, z-axis
    sigma_T : float
        extracellular conductivity in tissue slice
    sigma_G : float
        Conductivity of MEA glass electrode plane.
        Should normally be zero for MEA set up.
    sigma_S : float
        Conductivity of saline bath that tissue slice is immersed in
    steps : int
        Number of steps to average over the in technically infinite sum
    h : float
        Slice thickness in um.
    r_limit : np.ndarray
        minimum distance to source current for each compartment
    """

    dx2 = (x - cell.xmid)**2
    dy2 = (y - cell.ymid)**2
    dz2 = (z - cell.zmid)**2

    dL2 = dx2 + dy2
    inds = np.where(dL2 + dz2 < r_limit*r_limit)[0]
    dL2[inds] = r_limit[inds]*r_limit[inds] - dz2[inds]

    def _omega(dz):
        return 1/np.sqrt(dL2 + dz*dz)

    WTS = (sigma_T - sigma_S)/(sigma_T + sigma_S)
    WTG = (sigma_T - sigma_G)/(sigma_T + sigma_G)

    mapping = _omega(z - cell.zmid)
    mapping += (WTS * _omega(z + cell.zmid - 2*h) +
                WTG * _omega(z + cell.zmid))

    n = np.arange(1, steps)
    a = (WTS*WTG)**n[:, None] * (WTS * _omega(z + cell.zmid - 2*(n[:, None] + 1)*h) +
                                 WTG * _omega(z + cell.zmid + 2*n[:, None]*h) +
                                 _omega(z - cell.zmid + 2*n[:, None]*h) +
                                 _omega(z - cell.zmid - 2*n[:, None]*h))
    mapping += np.sum(a, axis=0)
    mapping *= 1/(4*np.pi*sigma_T)

    return mapping


def calc_lfp_linesource_moi(cell, x, y, z, sigma_T, sigma_S, sigma_G,
                             steps, h, r_limit, **kwargs):
    """Calculate extracellular potentials using the line-source
    equation on all compartments for in vitro Microelectrode Array (MEA) slices

    Parameters
    ----------
    cell: obj
        LFPy.Cell or LFPy.TemplateCell like instance
    x : float
        extracellular position, x-axis
    y : float
        extracellular position, y-axis
    z : float
        extracellular position, z-axis
    sigma_T : float
        extracellular conductivity in tissue slice
    sigma_G : float
        Conductivity of MEA glass electrode plane.
        Should normally be zero for MEA set up, and for this method,
        only zero valued sigma_G is supported.
    sigma_S : float
        Conductivity of saline bath that tissue slice is immersed in
    steps : int
        Number of steps to average over the in technically infinite sum
    h : float
        Slice thickness in um.
    r_limit : np.ndarray
        minimum distance to source current for each compartment
    """

    if np.abs(z) > 1e-9:
        raise RuntimeError("This method can only handle electrodes "
                           "at the MEA plane z=0")
    if np.abs(sigma_G) > 1e-9:
        raise RuntimeError("This method can only handle sigma_G=0, i.e.,"
                           "a non-conducting MEA glass electrode plane.")

    xstart = cell.xstart
    xend = cell.xend
    ystart = cell.ystart
    yend = cell.yend
    zstart = cell.zstart
    zend = cell.zend
    x0, y0, z0 = cell.xstart, cell.ystart, cell.zstart
    x1, y1, z1 = cell.xend, cell.yend, cell.zend

    pos = np.array([x, y, z])
    rs, closest_points = return_dist_from_segments(xstart, ystart, zstart,
                                                   xend, yend, zend, pos)
    z0_ = z0.copy()
    z0_[np.where(rs < r_limit)] = r_limit[np.where(rs < r_limit)]

    ds = _deltaS_calc(xstart, xend, ystart, yend, zstart, zend)
    factor_a = ds*ds
    dx = x1 - x0
    dy = y1 - y0
    dz = z1 - z0
    a_x = x - x0
    a_y = y - y0
    W = (sigma_T - sigma_S)/(sigma_T + sigma_S)
    num = np.zeros(factor_a.shape)
    den = np.zeros(factor_a.shape)

    def _omega(a_z):
        #See Rottman integration formula 46) page 137 for explanation

        factor_b = - a_x*dx - a_y*dy - a_z*dz
        factor_c = a_x*a_x + a_y*a_y + a_z*a_z
        b_2_ac = factor_b*factor_b - factor_a * factor_c

        case1_idxs = np.where(np.abs(b_2_ac) <= 1e-12)
        case2_idxs = np.where(np.abs(b_2_ac) > 1e-12)

        if not len(case1_idxs) == 0:
            num[case1_idxs] = factor_a[case1_idxs] + factor_b[case1_idxs]
            den[case1_idxs] = factor_b[case1_idxs]
        if not len(case2_idxs) == 0:
            num[case2_idxs] = (factor_a[case2_idxs] + factor_b[case2_idxs] +
                               ds[case2_idxs]*np.sqrt(factor_a[case2_idxs] +
                               2*factor_b[case2_idxs] + factor_c[case2_idxs]))
            den[case2_idxs] = (factor_b[case2_idxs] +
                               ds[case2_idxs]*np.sqrt(factor_c[case2_idxs]))
        return np.log(num/den)

    mapping = _omega(-z0_)
    n = 1
    while n < steps:
        mapping += W**n * (_omega(2*n*h - z0_) + _omega(-2*n*h - z0_))
        n += 1

    mapping *= 2/(4*np.pi*sigma_T * ds)

    return mapping


def calc_lfp_soma_as_point_moi(cell, x, y, z, sigma_T, sigma_S, sigma_G,
                             steps, h, r_limit, **kwargs):
    """Calculate extracellular potentials for in vitro
    Microelectrode Array (MEA) slices, where soma (compartment zero) is
    treated as a point source, and all other compartments as line sources.

    Parameters
    ----------
    cell: obj
        LFPy.Cell or LFPy.TemplateCell like instance
    x : float
        extracellular position, x-axis
    y : float
        extracellular position, y-axis
    z : float
        extracellular position, z-axis
    sigma_T : float
        extracellular conductivity in tissue slice
    sigma_G : float
        Conductivity of MEA glass electrode plane.
        Should normally be zero for MEA set up, and for this method,
        only zero valued sigma_G is supported.
    sigma_S : float
        Conductivity of saline bath that tissue slice is immersed in
    steps : int
        Number of steps to average over the in technically infinite sum
    h : float
        Slice thickness in um.
    r_limit : np.ndarray
        minimum distance to source current for each compartment
    """

    if np.abs(z) > 1e-9:
        raise RuntimeError("This method can only handle electrodes "
                           "at the MEA plane z=0")
    if np.abs(sigma_G) > 1e-9:
        raise RuntimeError("This method can only handle sigma_G=0, i.e.,"
                           "a non-conducting MEA glass electrode plane.")

    xstart = cell.xstart
    xend = cell.xend
    ystart = cell.ystart
    yend = cell.yend
    zstart = cell.zstart
    zend = cell.zend
    x0, y0, z0 = cell.xstart, cell.ystart, cell.zstart
    x1, y1, z1 = cell.xend, cell.yend, cell.zend

    pos = np.array([x, y, z])
    rs, closest_points = return_dist_from_segments(xstart, ystart, zstart,
                                                   xend, yend, zend, pos)
    z0_ = np.array(z0)
    if np.any(rs < r_limit):
        z0_[rs < r_limit] = r_limit


    ds = _deltaS_calc(xstart, xend, ystart, yend, zstart, zend)
    factor_a = ds*ds
    dx = x1 - x0
    dy = y1 - y0
    dz = z1 - z0
    a_x = x - x0
    a_y = y - y0
    W = (sigma_T - sigma_S)/(sigma_T + sigma_S)
    num = np.zeros(factor_a.shape)
    den = np.zeros(factor_a.shape)

    def _omega(a_z):
        #See Rottman integration formula 46) page 137 for explanation

        factor_b = - a_x*dx - a_y*dy - a_z*dz
        factor_c = a_x*a_x + a_y*a_y + a_z*a_z
        b_2_ac = factor_b*factor_b - factor_a * factor_c

        case1_idxs = np.where(np.abs(b_2_ac) <= 1e-12)
        case2_idxs = np.where(np.abs(b_2_ac) > 1e-12)

        if not len(case1_idxs) == 0:
            num[case1_idxs] = factor_a[case1_idxs] + factor_b[case1_idxs]
            den[case1_idxs] = factor_b[case1_idxs]
        if not len(case2_idxs) == 0:
            num[case2_idxs] = (factor_a[case2_idxs] + factor_b[case2_idxs] +
                               ds[case2_idxs]*np.sqrt(factor_a[case2_idxs] +
                               2*factor_b[case2_idxs] + factor_c[case2_idxs]))
            den[case2_idxs] = (factor_b[case2_idxs] +
                               ds[case2_idxs]*np.sqrt(factor_c[case2_idxs]))
        return np.log(num/den)

    mapping = _omega(-z0_)
    n = 1
    while n < steps:
        mapping += W**n * (_omega(2*n*h - z0) + _omega(-2*n*h - z0))
        n += 1

    mapping *= 2/(4*np.pi*sigma_T * ds)

    # NOW DOING SOMA

    # get compartment indices for somatic compartments (to be treated as point
    # sources)
    try:
        somainds = cell.get_idx("soma")
    except Exception:
        raise Exception('Call {}.get_idx("soma") failed in method LFPy.lfpcalc.calc_lfp_soma_as_point'.format(cell))

    dx2 = (x - cell.xmid[somainds])**2
    dy2 = (y - cell.ymid[somainds])**2
    dz2 = (z - cell.zmid[somainds])**2

    dL2 = dx2 + dy2
    inds = np.where(dL2 + dz2 < r_limit[somainds]*r_limit[somainds])[0]
    dL2[inds] = r_limit[inds]*r_limit[inds] - dz2[inds]

    def _omega(dz):
        return 1/np.sqrt(dL2 + dz*dz)

    mapping[somainds] = _omega(z - cell.zmid[somainds])
    mapping[somainds] += (W * _omega(cell.zmid[somainds] - 2*h) +
                       _omega(cell.zmid[somainds]))

    n = np.arange(1, steps)
    a = (W)**n[:, None] * (W * _omega(+ cell.zmid[somainds] - 2*(n[:, None] + 1)*h) +
                           2 * _omega(+ cell.zmid[somainds] + 2*n[:, None]*h) +
                               _omega(+ cell.zmid[somainds] - 2*n[:, None]*h))
    mapping[somainds] += np.sum(a, axis=0)
    mapping[somainds] *= 1/(4*np.pi*sigma_T)

    return mapping


#!/usr/bin/env python
"""
Copyright (C) 2012 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""

import os
import posixpath
import sys
import pickle
import numpy as np
import neuron
from LFPy import Cell, RecExtElectrode
from LFPy.run_simulation import _run_simulation, _run_simulation_with_electrode

class TemplateCell(Cell):

    """
    class LFPy.TemplateCell
    
    This class allow using NEURON templates with some limitations.

    This takes all the same parameters as the Cell class, but requires three
    more template related parameters
    
    Parameters
    ----------
    morphology : str
        path to morphology file
    templatefile : str
        File with cell template definition(s)
    templatename : str
        Cell template-name used for this cell object
    templateargs : str
        Parameters provided to template-definition
    v_init : float
        Initial membrane potential. Default to -65.
    Ra : float
        axial resistance. Defaults to 150.
    cm : float
        membrane capacitance. Defaults to 1.0
    passive : bool
        Passive mechanisms are initialized if True. Defaults to True
    passive_parameters : dict
        parameter dictionary with values for the passive membrane mechanism in
        NEURON ('pas'). The dictionary must contain keys 'g_pas' and 'e_pas',
        like the default: passive_parameters=dict(g_pas=0.001, e_pas=-70)
    extracellular : bool
        switch for NEURON's extracellular mechanism. Defaults to False
    dt: float
        Simulation time step. Defaults to 2**-4
    tstart : float
        initialization time for simulation <= 0 ms. Defaults to 0.
    tstop : float
        stop time for simulation > 0 ms. Defaults to 100.
    nsegs_method : 'lambda100' or 'lambda_f' or 'fixed_length' or None
        nseg rule, used by NEURON to determine number of compartments.
        Defaults to 'lambda100'
    max_nsegs_length : float or None
        max segment length for method 'fixed_length'. Defaults to None
    lambda_f : int
        AC frequency for method 'lambda_f'. Defaults to 100
    d_lambda : float
        parameter for d_lambda rule. Defaults to 0.1
    delete_sections : bool
        delete pre-existing section-references. Defaults to True
    custom_code : list or None
        list of model-specific code files ([.py/.hoc]). Defaults to None
    custom_fun : list or None
        list of model-specific functions with args. Defaults to None
    custom_fun_args : list or None
        list of args passed to custom_fun functions. Defaults to None
    pt3d : bool
        use pt3d-info of the cell geometries switch. Defaults to False
    celsius : float or None
        Temperature in celsius. If nothing is specified here
        or in custom code it is 6.3 celcius
    verbose : bool
        verbose output switch. Defaults to False

    Examples
    --------

    >>> import LFPy
    >>> cellParameters = {
    >>>     'morphology' : '<path to morphology.hoc>',
    >>>     'templatefile' :  '<path to template_file.hoc>'
    >>>     'templatename' :  'templatename'
    >>>     'templateargs' :  None
    >>>     'v_init' : -65,
    >>>     'cm' : 1.0,
    >>>     'Ra' : 150,
    >>>     'passive' : True,
    >>>     'passive_parameters' : {'g_pas' : 0.001, 'e_pas' : -65.},
    >>>     'dt' : 2**-3,
    >>>     'tstart' : 0,
    >>>     'tstop' : 50,
    >>> }
    >>> cell = LFPy.TemplateCell(**cellParameters)
    >>> cell.simulate()

    """

    def __init__(self,
                 templatefile='LFPyCellTemplate.hoc',
                 templatename='LFPyCellTemplate',
                 templateargs=None,
                 verbose=False,
                 **kwargs):
        """
        Initialization of the Template Cell object.

        """
        if "win32" in sys.platform and type(templatefile) is str:
            templatefile = templatefile.replace(os.sep, posixpath.sep)
        self.templatefile = templatefile
        self.templatename = templatename
        self.templateargs = templateargs
        self.verbose = verbose
        
        if not hasattr(neuron.h, 'd_lambda'):
            neuron.h.load_file('stdlib.hoc', 'String')    #NEURON std. library
            neuron.h.load_file('import3d.hoc')  #import 3D morphology lib
                
        #load the cell template specification
        #check if templatename exist in neuron.h namespace:
        if hasattr(neuron.h, self.templatename):
            if self.verbose:
                print('template %s exist already' % self.templatename)
        else:
            if type(self.templatefile) == str:
                neuron.h.load_file(self.templatefile)
            elif type(self.templatefile) == list:
                for template in self.templatefile:
                    if "win32" in sys.platform:
                        template = template.replace(os.sep, posixpath.sep)
                    neuron.h.load_file(template)
        
        #initialize the cell object
        Cell.__init__(self, **kwargs)

    def _load_geometry(self):
        """Load the morphology-file in NEURON""" 
        try: 
            neuron.h.sec_counted = 0
        except LookupError:
            neuron.h('sec_counted = 0')
                
        #the python cell object we are loading the morphology into:
        self.template = getattr(neuron.h, self.templatename)(self.templateargs)
        
        #perform a test if the morphology is already loaded:
        seccount = 0
        for sec in self.template.all:
            seccount += 1
        if seccount == 0:
            #import the morphology, try and determine format
            fileEnding = self.morphology.split('.')[-1]
        
            if not fileEnding == 'hoc' or fileEnding == 'HOC':            
                #create objects for importing morphologies of different formats
                if fileEnding == 'asc' or fileEnding == 'ASC':
                    Import = neuron.h.Import3d_Neurolucida3()
                    if not self.verbose:
                        Import.quiet = 1
                elif fileEnding == 'swc' or fileEnding ==  'SWC':
                    Import = neuron.h.Import3d_SWC_read()
                elif fileEnding == 'xml' or fileEnding ==  'XML':
                    Import = neuron.h.Import3d_MorphML()
                else:
                    raise ValueError('%s is not a recognised morphology file format! ').with_traceback('Should be either .hoc, .asc, .swc, .xml!' \
                         % self.morphology)
                
                #assuming now that morphology file is the correct format
                try:
                    Import.input(self.morphology)
                except:
                    if not hasattr(neuron, 'neuroml'):
                        raise Exception('Can not import, try and copy the ' + \
                        'nrn/share/lib/python/neuron/neuroml ' + \
                        'folder into %s' % neuron.__path__[0])
                    else:
                        raise Exception('something wrong with file, see output')
                try:
                    imprt = neuron.h.Import3d_GUI(Import, 0)
                except:
                    raise Exception('See output, try to correct the file')
                
                #instantiate the cell object
                if fileEnding == 'xml' or fileEnding ==  'XML':
                    #can not currently assign xml to cell template
                    try:
                        imprt.instantiate(self.template)
                    except:
                        raise Exception("this xml file is not supported")
                else:
                    imprt.instantiate(self.template)
                
            else:
                neuron.h.execute("xopen(\"%s\")" % self.morphology, self.template)
        
        #set shapes and create sectionlists
        neuron.h.define_shape()
        self._create_sectionlists()

    def _create_sectionlists(self):
        """Create section lists for different kinds of sections"""
        
        self.allsecnames = []
        for sec in self.template.all:
            self.allsecnames.append(sec.name())
        
        self.allseclist = self.template.all
        
        #list of soma sections, assuming it is named on the format "soma*"
        self.nsomasec = 0
        self.somalist = neuron.h.SectionList()
        for sec in self.allseclist:
            if 'soma' in sec.name():
                self.somalist.append(sec=sec)
                self.nsomasec += 1





#!/usr/bin/env python
'''
This is a scripts allowing the user to specify the path to a morphology file,
and determine the rotation angles that will rotate the apical dendrite along the
vertical z-axis. If desired, it will create a .rot-file that LFPy will
automatically use to set the default rotation alongside the morphology.
'''

#import some stuff
import pylab as pl
import LFPy
import os

#plot will pop up by itself
pl.interactive(1)

'''
Define some functions for plotting
'''
def plot_linepiece(ax, cell, i, color):
    ax.plot([cell.xstart[i], cell.xend[i]],
        [cell.ystart[i], cell.yend[i]],
        [cell.zstart[i], cell.zend[i]], color = color, lw = cell.diam[i])


def plot_morpho_indices(cell, new_fig = True):
    from mpl_toolkits.mplot3d import Axes3D
    if new_fig:
        fig = pl.figure(figsize=[10, 10])
    else:
        fig = pl.gcf()
    ax = Axes3D(fig)
    if cell.get_idx(section='apic').size > 0:
        apic = cell.get_idx(section='apic')
    else:
        apic = cell.get_idx(section='dend')
    for i in range(apic.size):
        try:
            ax.text(cell.xmid[apic[i]],cell.ymid[apic[i]],cell.zmid[apic[i]], 
                str(apic[i]), fontsize=8)
        except TypeError:
            ax.text(cell.xmid[apic[i]],cell.ymid[apic[i]],cell.zmid[apic[i]], 
                str(apic[i]))
    
    apic = cell.get_idx(section='apic')
    soma = cell.get_idx(section='soma')
    axon = cell.get_idx(section='axon')
    dend = cell.get_idx(section='dend')
    
    for i in soma:
        plot_linepiece(ax, cell, i, 'k')
    for i in dend:
        plot_linepiece(ax, cell, i, 'g')
    for i in axon:
        plot_linepiece(ax, cell, i, 'grey')
    for i in apic:
        plot_linepiece(ax, cell, i, 'm')
    

    ax.set_xlabel('x [$\mu$m]')
    ax.set_ylabel('y [$\mu$m]')
    ax.set_zlabel('z [$\mu$m]')

    #lim_3d = max([max(abs(cell.xmid[apic])), \
    #                    max(abs(cell.ymid[apic])), \
    #                    max(abs(cell.zmid[apic]))])
    #lim_3d = round(lim_3d/2,-2)
    #
    #ax.set_xlim3d(-lim_3d,lim_3d)
    #ax.set_ylim3d(-lim_3d,lim_3d)
    #ax.set_zlim3d(-lim_3d,lim_3d)
    
    ax.axis(ax.axis('equal'))
    

def plot_morpho_3D_simple(cell, new_fig = True, color = 'k'):
    from mpl_toolkits.mplot3d import Axes3D
    if new_fig:
        fig = pl.figure(figsize=[10, 10])
    else:
        fig = pl.gcf()
    ax = Axes3D(fig)
    apic = cell.get_idx(section='apic')
    soma = cell.get_idx(section='soma')
    axon = cell.get_idx(section='axon')
    dend = cell.get_idx(section='dend')
    
    for i in soma:
        plot_linepiece(ax, cell, i, 'k')
    for i in dend:
        plot_linepiece(ax, cell, i, 'g')
    for i in axon:
        plot_linepiece(ax, cell, i, 'grey')
    for i in apic:
        plot_linepiece(ax, cell, i, 'm')

    ax.set_xlabel('x [$\mu$m]')
    ax.set_ylabel('y [$\mu$m]')
    ax.set_zlabel('z [$\mu$m]')

    lim_3d = max([max(abs(cell.xstart)), max(abs(cell.xend)), \
                        max(abs(cell.ystart)), max(abs(cell.yend)), \
                        max(abs(cell.zstart)), max(abs(cell.zend))])
    lim_3d = round(lim_3d,-2)

    ax.set_xlim3d(-lim_3d,lim_3d)
    ax.set_ylim3d(-lim_3d,lim_3d)
    ax.set_zlim3d(-lim_3d,lim_3d)

def det_rotationangles2(cell,segment1,segment2):
    '''return rotationangles around x- and y-axis, so the line between two
    chosen segments is aligned in parallell to the z-axis, corrected to soma.'''
    R = pl.sqrt((cell.xmid[segment1]-cell.xmid[segment2])**2 \
            + (cell.ymid[segment1]-cell.ymid[segment2])**2 \
            + (cell.zmid[segment1]-cell.zmid[segment2])**2)

    rot_x = pl.pi+pl.arctan2(cell.ymid[segment1]-cell.ymid[segment2],\
                                   cell.zmid[segment1]-cell.zmid[segment2])
    rot_y = -pl.pi+pl.arcsin((cell.xmid[segment1]-cell.xmid[segment2])/R)

    rotation = {
        'x' : rot_x,
        'y' : rot_y
        }
    return rotation


'''
MAIN SCRIPT
'''

pl.close('all')

morphologyPath = input('Enter path to morphology file: ')
print('You entered %s.' % morphologyPath)

cellparams = {
    'morphology' : morphologyPath,
    'nsegs_method' : 'lambda_f',
    'lambda_f' : 10,
}

rotationFilePath = morphologyPath[:-4]+'.rot'

if os.path.isfile(rotationFilePath):
    print(('It appear as though there exist a file %s already' % rotationFilePath))
    print('I don\'t know what to do with all of this...')
    print('It\'s soo depressive....')
else:
    print('Will create file %s with the rotation information' % rotationFilePath)

    ok = 'n'

    while ok != 'y':

        cell = LFPy.Cell(**cellparams)

        plot_morpho_indices(cell)

        print('Identify TWO indices to be aligned with the z-axis in the plot')
        
        [a, b] = eval(input('Type them in here (sep. by comma): '))

        r_a = pl.sqrt(cell.xmid[a]**2 + cell.ymid[a]**2 + cell.zmid[a]**2)
        r_b = pl.sqrt(cell.xmid[b]**2 + cell.ymid[b]**2 + cell.zmid[b]**2)

        if r_a < r_b:
            b1 = a
            a1 = b
        else:
            a1 = a
            b1 = b

        rotation = det_rotationangles2(cell, segment1=a1,segment2=b1)
        print('Determined rotation angles to be (in radians): ', rotation)

        cell.set_rotation(**rotation)

        plot_morpho_3D_simple(cell)

        ok = input('Does it look correctly rotated (y)? ')
        print(ok)

    printtofile = input('Shall I write the file %s then (y) ? ' % rotationFilePath)
    if printtofile == 'y':
        F = file(rotationFilePath,'w')

        F.write('x=%g\n' % rotation['x'])
        F.write('y=%g\n' % rotation['y'])
        F.write('z=%g\n' % 0)

        F.close()
        
        print('file %s created!' % rotationFilePath)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
LFPs from a population of cells relying on MPI (Message Passing Interface)

Execution:

    <mpiexec> -n <processes> python example_mpi_2.py

Notes:
- on certain platforms and with mpirun, the --oversubscribe argument is needed
  to get more processes than the number of physical CPU cores.

Copyright (C) 2017 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
"""

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.collections import PolyCollection, LineCollection
import os
from os.path import join
import sys
if sys.version < '3':
    from urllib2 import urlopen
else:
    from urllib.request import urlopen
import zipfile
import LFPy
import neuron
from mpi4py import MPI

#initialize the MPI interface
COMM = MPI.COMM_WORLD
SIZE = COMM.Get_size()
RANK = COMM.Get_rank()

#set the numpy random seeds
global_seed = 1234
np.random.seed(global_seed)

def stationary_poisson(nsyn,lambd,tstart,tstop):
    ''' Generates nsyn stationary possion processes with rate lambda between tstart and tstop'''
    interval_s = (tstop-tstart)*.001
    spiketimes = []
    for i in range(nsyn):
        spikecount = np.random.poisson(interval_s*lambd)
        spikevec = np.empty(spikecount)
        if spikecount==0:
            spiketimes.append(spikevec)
        else:
            spikevec = tstart + (tstop-tstart)*np.random.random(spikecount)
            spiketimes.append(np.sort(spikevec)) #sort them too!

    return spiketimes


#Fetch Mainen&Sejnowski 1996 model files
if not os.path.isfile(join('cells', 'cells', 'j4a.hoc')) and RANK==0:
    #get the model files:
    u = urlopen('http://senselab.med.yale.edu/ModelDB/eavBinDown.asp?o=2488&a=23&mime=application/zip')
    localFile = open('patdemo.zip', 'w')
    localFile.write(u.read())
    localFile.close()
    #unzip:
    myzip = zipfile.ZipFile('patdemo.zip', 'r')
    myzip.extractall('.')
    myzip.close()

#resync MPI threads
COMM.Barrier()

# Define cell parameters
cell_parameters = {          # various cell parameters,
    'morphology' : join('cells', 'cells', 'j4a.hoc'), # Mainen&Sejnowski, 1996
    'cm' : 1.0,         # membrane capacitance
    'Ra' : 150,         # axial resistance
    'v_init' : -65.,    # initial crossmembrane potential
    'passive' : True,   # turn on passive mechanism for all sections
    'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65}, # passive params
    'nsegs_method' : 'lambda_f',
    'lambda_f' : 100.,
    'dt' : 2.**-3,      # simulation time step size
    'tstart' :  0.,     # start time of simulation, recorders start at t=0
    'tstop' : 300.,     # stop simulation at 200 ms. These can be overridden
                        # by setting these arguments i cell.simulation()
}

# Define synapse parameters
synapse_parameters = {
    'idx' : 0, # to be set later
    'e' : 0.,                   # reversal potential
    'syntype' : 'ExpSyn',       # synapse type
    'tau' : 5.,                 # syn. time constant
    'weight' : .001,            # syn. weight
    'record_current' : True,
}

# Define electrode parameters
point_electrode_parameters = {
    'sigma' : 0.3,      # extracellular conductivity
    'x' : 0.,  # electrode requires 1d vector of positions
    'y' : 0.,
    'z' : 0.,
}

# number of units
n_cells = 6

# assign cell positions
x_cell_pos = np.linspace(-250., 250., n_cells)

# default rotation around x and y axis
xy_rotations = dict(x=4.99, y=-4.33)

# rotations around z-axis
if RANK == 0:
    z_rotation = COMM.bcast(np.random.permutation(np.arange(0., np.pi,
                                                            np.pi / n_cells)),
                            root=0)
else:
    z_rotation = COMM.bcast(None, root=0)


#synaptic spike times drawn on RANK 0 distributed to all processes
n_pre_syn = 1000
if RANK == 0:
    pre_syn_sptimes = COMM.bcast(stationary_poisson(nsyn=n_pre_syn, lambd=5.,
                                                    tstart=0, tstop=300),
                                 root=0)
else:
    pre_syn_sptimes = COMM.bcast(None, root=0)

# number of synapses on each cell
n_synapses = 100

# indices for presynaptic spike trains for each neuron also picked on RANK 0
# and scattered (for illustrating purposes, not efficiency)
if RANK == 0:
    # set up len SIZE nested list for spike train IDs.
    pre_syn_ids = [[]]*SIZE
    for cell_id in range(n_cells):
        pre_syn_ids[cell_id % SIZE] += [np.random.permutation(np.arange(
                                                    n_pre_syn))[0:n_synapses]]
else:
    pre_syn_ids = None
pre_syn_ids = COMM.scatter(pre_syn_ids, root=0)

# containers for per-cell LFP and summed LFPs
single_LFPs = []
summed_LFP = np.zeros(int(cell_parameters['tstop'] / cell_parameters['dt'] + 1))

# get state of random seed generator
state = np.random.get_state()

# iterate over cells in populations
for cell_id in range(n_cells):
    if cell_id % SIZE == RANK:
        # get set seed per cell in order to synapse locations
        np.random.seed(global_seed + cell_id)

        # Create cell
        cell = LFPy.Cell(**cell_parameters)

        #Have to position and rotate the cells!
        cell.set_rotation(z=z_rotation[cell_id], **xy_rotations)
        cell.set_pos(x=x_cell_pos[cell_id])

        for i_syn in range(n_synapses):
            syn_idx = cell.get_rand_idx_area_norm()
            synapse_parameters.update({'idx' : syn_idx})
            synapse = LFPy.Synapse(cell, **synapse_parameters)
            synapse.set_spike_times(pre_syn_sptimes[pre_syn_ids[
                                                        cell_id % SIZE][i_syn]])

        #run the cell simulation
        cell.simulate(rec_imem=True)

        #set up the extracellular device
        point_electrode = LFPy.RecExtElectrode(cell,
                                               **point_electrode_parameters)
        point_electrode.calc_lfp()

        # sum LFP on this RANK
        summed_LFP += point_electrode.LFP[0]

        # send LFP of this cell to RANK 0
        if RANK != 0:
            COMM.send(point_electrode.LFP[0], dest=0)
        else:
            single_LFPs += [point_electrode.LFP[0]]

    # collect single LFP contributions on RANK 0
    if RANK == 0:
        if cell_id % SIZE != RANK:
            single_LFPs += [COMM.recv(source=cell_id % SIZE)]

# we can also use MPI to sum arrays directly:
summed_LFP = COMM.reduce(summed_LFP)

# reset state of random number generator
np.random.set_state(state)

# plot output on RANK 0.
if RANK==0:
    #assign color to each unit
    color_vec = [plt.cm.rainbow(int(x*256./n_cells)) for x in range(n_cells)]

    #figure
    fig = plt.figure(figsize=(12, 8))

    # Morphologies axes:
    plt.axes([.175, .0, .65, 1], aspect='equal')
    plt.axis('off')

    for i_cell in range(n_cells):
        cell = LFPy.Cell(join('cells', 'cells', 'j4a.hoc'),
                         nsegs_method='lambda_f',
                         lambda_f=5)
        cell.set_rotation(z=z_rotation[i_cell], **xy_rotations)
        cell.set_pos(x=x_cell_pos[i_cell])

        zips = []
        for x, z in cell.get_idx_polygons():
            zips.append(list(zip(x, z)))
        linecol = LineCollection(zips,
                    edgecolor = 'none',
                    facecolor = color_vec[i_cell],
                    rasterized=False,
                    )

        ax = plt.gca()
        ax.add_collection(linecol)

    axis = ax.axis(ax.axis('equal'))
    ax.axis(np.array(axis) / 1.15)


    #adding a blue dot:
    ax.plot(point_electrode.x, point_electrode.z, 'o',
            markeredgecolor='none', markerfacecolor='b', markersize=3,
            zorder=10, clip_on=False)
    plt.annotate("Electrode",
            xy=(0., 0.), xycoords='data',
            xytext=(-100., 1000.),
            arrowprops=dict(arrowstyle='wedge',
                            shrinkA=1,
                            shrinkB=1,
                            #lw=0.5,
                            mutation_scale=20,
                            fc="0.6", ec="none",
                            edgecolor='k', facecolor='w'))

    plt.xlim([-700., 700.])

    ax.plot([100, 200], [-250, -250], 'k', lw=1, clip_on=False)
    ax.text(150, -300, r'100$\mu$m', va='center', ha='center')

    #presynaptic spike trains axes
    plt.axes([.05, .35, .25, .55])

    pop_sptimes = []
    for i_pre in range(n_pre_syn):
        sp = pre_syn_sptimes[i_pre]
        for i_sp in range(len(sp)):
            pop_sptimes.append(sp[i_sp])

    for i_pre in range(n_pre_syn):
        plt.scatter(pre_syn_sptimes[i_pre],
                    i_pre*np.ones(len(pre_syn_sptimes[i_pre])),
                    s=1, edgecolors='none', facecolors='k')

    plt.ylim([0,n_pre_syn])
    plt.xlim([0,cell_parameters['tstop']])
    plt.ylabel('train #', ha='left', labelpad=0)
    plt.title('Presynaptic spike times')

    ax = plt.gca()
    for loc, spine in ax.spines.items():
        if loc in ['right', 'top']:
            spine.set_color('none')
    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.set_ticks_position('left')

    ax.set_xticklabels([])

    #spike rate axes
    plt.axes([.05,.12,.25,.2])

    binsize = 5
    bins=np.arange(0, cell_parameters['tstop']+1., binsize)
    count,b = np.histogram(pop_sptimes, bins=bins)
    rate = count*(1000./binsize)*(1./n_pre_syn)
    plt.plot(b[0:-1],rate,color='black',lw=1)

    plt.xlim([0,cell_parameters['tstop']])
    plt.ylim([0,10.])

    tvec = np.arange(point_electrode.LFP.shape[1])*cell.dt

    plt.xlabel('$t$ (ms)')
    plt.ylabel('Rate (spike/s)')

    ax = plt.gca()
    for loc, spine in ax.spines.items():
        if loc in ['right', 'top']:
            spine.set_color('none')
    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.set_ticks_position('left')

    #single neuron EPs axes
    plt.axes([.7,.35,.25,.55])

    plt.title('Single neuron extracellular potentials')
    plt.axis('off')

    for cell_id in range(n_cells):
        plt.plot(tvec,
                        cell_id+2.e3*single_LFPs[cell_id],
                        color=color_vec[cell_id], lw=1,
                        )

    plt.ylim([-1,n_cells-.5])

    #Summed LFPs axes
    plt.axes([.7,.12,.25,.2])
    plt.plot(tvec, 1E3*summed_LFP, color='black', lw=1)
    plt.ylim([-5.e-1,5e-1])

    plt.title('Summed extracellular potentials')
    plt.xlabel(r'$t$ (ms)')
    plt.ylabel(r'$\mu$V',ha='left',rotation='horizontal')

    ax = plt.gca()
    for loc, spine in ax.spines.items():
        if loc in ['right', 'top']:
            spine.set_color('none')
    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.set_ticks_position('left')


    fig.savefig('example_mpi_2.pdf', dpi=300)
    plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Defines parameters for simulation, used by example_parallel_network.py script

Copyright (C) 2018 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
'''
from __future__ import division
import matplotlib
import os
if not 'DISPLAY' in os.environ.keys():
    matplotlib.use('agg')
import os
import numpy as np
from scipy import stats
from glob import glob
import json
from parameters import ParameterSet
from mpi4py import MPI
import neuron
import sys
if sys.version < '3':
    stringType = 'S' # fix execution on Python 2 and 3
    from urllib2 import urlopen
else:
    stringType = 'U'
    from urllib.request import urlopen
from example_parallel_network_methods import get_templatename, get_params, get_syn_params
import LFPy

# set up MPI environment
COMM = MPI.COMM_WORLD
SIZE = COMM.Get_size()
RANK = COMM.Get_rank()


# load some neuron-interface files needed for the EPFL cell types
neuron.h.load_file("stdrun.hoc")
neuron.h.load_file("import3d.hoc")


#######################
# Functions
#######################
def get_pre_m_type(post):
    '''little helper function to return the most populuous excitatory m_type
    within the layer of m_type post, assuming this is representative for
    excitatory external connections onto postsynaptic cells '''
    if post.startswith('L23'):
        return 'L23_PC'
    elif post.startswith('L4'):
        return 'L4_PC'
    elif post.startswith('L5'):
        return 'L5_TTPC1'
    elif post.startswith('L6'):
        return 'L6_IPC'

#######################
# Parameters
#######################


# test mode (1 cell per pop, all-to-all connectivity)
TESTING = False

# Creating a NeuroTools.parameters.ParameterSet object for the main parameters
PSET = ParameterSet({})

# output file destination
if TESTING:
    PSET.OUTPUTPATH = 'example_parallel_network_output_testing'
else:
    PSET.OUTPUTPATH = 'example_parallel_network_output'

# input file paths
# PATHs to current cell-specific files and NMODL files
PSET.CWD = os.getcwd()
PSET.CELLPATH = 'hoc_combos_syn.1_0_10.allzips'
PSET.NMODL = 'hoc_combos_syn.1_0_10.allmods'


########################################################
# Simulation control
########################################################
PSET.dt = 2**-4             # simulation time step size (ms)
PSET.tstop = 1500.          # simulation duration (ms)
PSET.v_init = -77.          # membrane voltage(s) at t = 0 for all cells
PSET.celsius = 34.          # global temperature setting affecting active channels
PSET.TRANSIENT = 500.       # duration of startup transient

# population size scaling (multiplied with values in populationParams['POP_SIZE']):
PSET.POPSCALING = 1. 

# global scaling of connection probabilities (to counteract POPSCALING)
PSET.CONNPROBSCALING = 1. / PSET.POPSCALING

# switch for fully connected network (do not use with large population sizes)
PSET.fully_connected = True if TESTING else False

# bool flag switching LFP calculations on or off (faster)
PSET.COMPUTE_LFP = True

# bool flag switching ECoG calculation on or off
PSET.COMPUTE_ECOG = PSET.COMPUTE_LFP

# bool flag switching on calculations of electric current dipole moments per population
PSET.COMPUTE_P = PSET.COMPUTE_LFP

# bool flag switching on calculations of contributions to the extracellular
# potential per population
PSET.rec_pop_contributions = PSET.COMPUTE_LFP

# downsample factor for timeseries plots
PSET.decimate_q = 10

# settings for filtered signals shown in plots (fc=100 Hz, lowpass)
PSET.filterargs=dict(N=2, Wn=100.*2.*PSET.dt/1000*PSET.decimate_q, btype='lowpass')

# Base NetworkCell arguments, morphology and template specific args is
# defined below. 
cellParams = {
    'passive' :     False,
    'nsegs_method' : None,
    'v_init' : PSET.v_init,
    'tstart' : 0,
    'tstop' : PSET.tstop,
    'dt' : PSET.dt,
    'verbose' : False,
    'extracellular' : False,
    'delete_sections' : False,
}


#some stimuli to activate the network
PSET.PointProcParams = {
    'idx' : 0,
    'record_current' : False,
    'pptype' : 'IClamp',
    # 'amp' : 0.793, # amplitude parameter set later on
    'dur' : 1E6,
    'delay' : 0.,
}

# parameters for predicting extracellular potentials, specifying
# coordinates of electrodes and extracellular conductivity. So far only
# point contacts
PSET.electrodeParams = {
    'x' : np.zeros(16),
    'y' : np.zeros(16),
    'z' : np.linspace(-1500, 0, 16)[::-1],
    'sigma' : 0.3,
    'n' : 50,
    'N' : np.array([[0., 1., 0]]*16),
    'r' : 5.,
    'method' : 'soma_as_point',
}


# parameters for 4-sphere volume conductor model
# compute electrode positions on the outer radius for different angular offsets
_theta = np.linspace(-np.pi/4, np.pi / 4, 9)
_x = 90000.*np.sin(_theta)
_y = np.zeros(_theta.size)
_z = 90000.*np.cos(_theta)
PSET.foursphereParams = {
    'radii' : [79000., 80000., 85000., 90000.], # shell radii
    'sigmas' : [0.3, 1.5, 0.015, 0.3],          # shell conductivity
    'r' : np.c_[_x, _y, _z],                    # contact coordinates
}


# Optional arguments to Network.simulate() for computing extracellular potential
# contribution from passive leak, membrane capactitance and synaptic currents
PSET.NetworkSimulateArgs = {
    'use_ipas' : False,
    'use_icap' : False,
    'use_isyn' : False,
}

# layer thickness top to bottom L1-L6, Markram et al. 2015 Fig 3A. 
PSET.layer_data = np.array([('L1', 165., -82.5),
                            ('L2', 149., -239.5),
                            ('L3', 353., -490.5),
                            ('L4', 190., -762.),
                            ('L5', 525,  -1119.5),
                            ('L6', 700,  -1732.)],
    dtype=[('layer', '|{}2'.format(stringType)), ('thickness', float), ('center', float)])



# Define electrode geometry corresponding to an ECoG electrode, where contact
# points have a radius r, surface normal vectors N, and ECoG is calculated as the
# average LFP in n random points on each contact:
PSET.ecogParameters = {
    'sigma_S' : 0.,        # CSF conductivity            
    'sigma_T' : 0.3,        # GM conductivity
    'sigma_G' : 0.3,        # WM conductivity
    'h': PSET.layer_data['thickness'].sum(),
    'x' : np.array([0.]),    # x,y,z-coordinates of electrode contacts
    'y' : np.array([0.]),
    'z' : np.array([0.]),
    'z_shift' : -PSET.layer_data['thickness'].sum(), #+PSET.layer_data[4]['thickness']/8,
    'n' : 500,
    'r' : 250, # ECoG radii are often 500-1000 um
    'N' : np.array([[0., 0., 1.]]),
    'method': "pointsource",
}


# Main population parameters:
PSET.populationParameters = np.array([
        # Layer 4 
        # Excitatory
        ('L4_PC',       'cAD', 'L4_PC_cADpyr230_1',        2674,
         dict(radius=210, loc=PSET.layer_data[3]['center'], scale=100., cap=[1078., 97.]),
         dict(x=np.pi/2, y=0.), ['dend', 'apic'],            ['dend', 'apic'],   0.125,    5.),
        # Inhibitory
        ('L4_LBC',      'dNAC', 'L4_LBC_dNAC222_1',         122,
         dict(radius=210, loc=PSET.layer_data[3]['center'], scale=100., cap=[938., 670]),
         dict(x=np.pi/2, y=0.), ['soma', 'dend', 'apic'],    ['dend', 'apic'],   0.125,    5.),
        
        # Layer 5
        # Excitatory
        ('L5_TTPC1',    'cAD', 'L5_TTPC1_cADpyr232_1',     2403,
         dict(radius=210, loc=PSET.layer_data[4]['center'], scale=125., cap=[719, 73.]),
         dict(x=np.pi/2, y=0.), ['dend', 'apic'],            ['dend', 'apic'],   0.1,   5.),
        # Inhibitory
        ('L5_MC',       'bAC', 'L5_MC_bAC217_1',            395,
         dict(radius=210, loc=PSET.layer_data[4]['center'], scale=125., cap=[378., 890]),
         dict(x=np.pi/2, y=0.), ['soma', 'dend', 'apic'],    ['dend', 'apic'],   0.125,    5.),
        ],
    dtype = [('m_type', '|{}32'.format(stringType)), ('e_type', '|{}32'.format(stringType)),
             ('me_type', '|{}32'.format(stringType)), ('POP_SIZE', 'i8'), ('pop_args', dict),
             ('rotation_args', dict), ('syn_section', list),
             ('extrinsic_input_section', list), ('extrinsic_input_density', 'f8'),
             ('extrinsic_input_frequency', 'f8')])
# column data:
# shortnames as used in pathway_*.json files
# names as used to denote individual cell types
# POP_SIZE :    number of neurons for each morphological type as given on https://bbp.epfl.ch/nmc-portal/microcircuit 


# pop_args : dict, radius, mean position (loc) and standard deviation (scale) of the soma positions
# rotation_args : dict, default rotations around x and y axis applied to each cell in the population using LFPy.NetworkCell.set_rotation() method. 

# syn_section : list of section names where outgoing connections from this population are made onto postsynaptic neurons (i.e., no excitatory synapses on somatic sections anywhere)
# extrinsic_input_density : density of extrinisc incoming connections in units of [µm^-2] 
# extrinsic_input_frequency : frequency of synapse activation in units of [Hz]

# TODO: Define only short names, pick random cell types or similar when creating populations. Column could be redone as [('m_type', '|U8'), ('e-type', '|U8')] and
# single cell objects picked from the glob('m+e type') on random

# # Override population sizes (for testing)
if TESTING:
    PSET.populationParameters['POP_SIZE'] = np.ones(PSET.populationParameters.size)


# Define a layer-specificity of connections L_YXL
# (see Hagen, Dahmen et al. (2016), Cereb Cortex) based on the anatomy of
# dendrites and axons. We here define this depth-dependence of synapse
# positioning as the product of total [soma + dendrite] length and
# total axon length in spatial bins corresponding to the thickness and
# boundaries of each layer. The products are normalized such that the sum of
# each column is 1, i.e., the sum of layer specificities of a connection
# between X and Y is 1.
PSET.L_YXL_m_types = {}
bins = np.r_[-PSET.layer_data['thickness'].cumsum()[::-1], 0]
for i, (y, Y, pop_args_Y, rotation_args_Y) in enumerate(zip(PSET.populationParameters['m_type'],
                                                            PSET.populationParameters['me_type'],
                                                            PSET.populationParameters['pop_args'],
                                                            PSET.populationParameters['rotation_args'])):
    # create a container for the layer specificities of connections
    data = np.zeros((PSET.layer_data.size,
                     PSET.populationParameters.size))
    
    # find and load the corresponding morphology files into LFPy
    m_Y = glob(os.path.join(PSET.CELLPATH, Y, 'morphology', '*.asc'))[0]
    cell_Y = LFPy.Cell(morphology=m_Y)
    cell_Y.set_rotation(**rotation_args_Y)
    cell_Y.set_pos(z=pop_args_Y['loc'])
    
    # sum the total length of axon in each layer bin
    layerbounds = np.r_[0, -PSET.layer_data['thickness'].cumsum()]
    len_Y_sum = np.zeros(PSET.layer_data.size)
    for k in range(PSET.layer_data.size):
        len_Y_sum[k] = cell_Y.length[cell_Y.get_idx(['soma', 'dend', 'apic'], z_min=layerbounds[k+1], z_max=layerbounds[k])].sum()
    for j, (X, pop_args_X, rotation_args_X) in enumerate(zip(PSET.populationParameters['me_type'],
                                                             PSET.populationParameters['pop_args'],
                                                             PSET.populationParameters['rotation_args'])):
        m_X = glob(os.path.join(PSET.CELLPATH, X, 'morphology', '*.asc'))[0]
        cell_X = LFPy.Cell(morphology=m_X)
        cell_X.set_rotation(**rotation_args_X)
        cell_X.set_pos(z=pop_args_X['loc'])
        
        len_X_sum = np.zeros(PSET.layer_data.size)
        for k in range(PSET.layer_data.size):
            len_X_sum[k] = cell_X.length[cell_X.get_idx('axon', z_min=layerbounds[k+1], z_max=layerbounds[k])].sum()
        
        data[:, j] = np.sqrt(len_Y_sum * len_X_sum) / np.sqrt(len_Y_sum * len_X_sum).sum()
    # fill in 
    PSET.L_YXL_m_types[y] = data
    
# clean up namespace
del cell_X, cell_Y, len_X_sum, len_Y_sum, data


# Container for LFPy.NetworkCell class parameters (path to morphology file etc.)
PSET.cellParameters = dict()

#################################################################################
# Set up various files and folders such that single-cell models from BBP can be used,
# and extract some numbers from pathway .json files
#################################################################################

# TODO: Add automated download of cell models from EPFL microcircuit portal


# autodownload some json files with anatomical and pathway specific data
pathway_files = ['pathways_anatomy_factsheets_simplified.json',
                 'pathways_physiology_factsheets_simplified.json']
if RANK == 0:
    for fname in pathway_files:            
        if not os.path.isfile(fname):
            u = urlopen('https://bbp.epfl.ch/nmc-portal/documents/10184/7288948/'+fname)
            localFile = open(fname, 'w')
            localFile.write(u.read().decode('utf-8'))
            localFile.close()
            u.close()
COMM.Barrier()

#flag for cell template file to switch on (inactive) synapses
add_synapses = False

# load synapse file info for each cell type as structured arrays in dictionary
synapses_tsv_dtype = [
    ('synapse_id', int),
    ('pre_cell_id', int),
    ('pre_mtype', int),
    ('sectionlist_id', int),
    ('sectionlist_index', int),
    ('seg_x', float),
    ('synapse_type', int),
    ('dep', float),
    ('fac', float),
    ('use', float),
    ('tau_d', float),
    ('delay', float),
    ('weight', float)
    ]
synapses_tsv = {}


# attempt to set up a folder with all unique EPFL mechanism mod files,
# compile, and load them all in order to be able to load cells as LFPy.NetworkCell objects 
if RANK == 0:
    if not os.path.isdir(PSET.NMODL):
        os.mkdir(PSET.NMODL)
        for NRN in PSET.populationParameters['me_type']:
            for nmodl in glob(os.path.join(PSET.CELLPATH, NRN, 'mechanisms', '*.mod')):
                while not os.path.isfile(os.path.join(PSET.NMODL,
                                                      os.path.split(nmodl)[-1])):
                    os.system('cp {} {}'.format(nmodl,
                                                os.path.join(PSET.NMODL,
                                                             '.')))
        os.chdir(PSET.NMODL)
        # patch faulty ProbGABAAB_EMS.mod file (otherwise stochastic inhibitory synapses will stay closed except at first activation)
        diff = '''319c319
<                 urand = scop_random(1)
---
>                 value = scop_random(1)
'''
        f = open('ProbGABAAB_EMS.patch', 'w')
        f.writelines(diff)
        f.close()
        os.system('patch ProbGABAAB_EMS.mod ProbGABAAB_EMS.patch')
        
        os.system('nrnivmodl')
        os.chdir(PSET.CWD)
COMM.Barrier()
neuron.load_mechanisms(PSET.NMODL)
os.chdir(PSET.CWD)


# Fill in dictionary of population-specific cell parameters
for NRN in PSET.populationParameters['me_type']:
    os.chdir(os.path.join(PSET.CWD, PSET.CELLPATH, NRN))
    
    #get the template name
    f = open("template.hoc", 'r')
    templatename = get_templatename(f)
    f.close()
    
    #get biophys template name
    f = open("biophysics.hoc", 'r')
    biophysics = get_templatename(f)
    f.close()
    
    #get morphology template name
    f = open("morphology.hoc", 'r')
    morphology = get_templatename(f)
    f.close()
    
    #get synapses template name
    f = open(os.path.join("synapses", "synapses.hoc"), 'r')
    synapses = get_templatename(f)
    f.close()

    if not hasattr(neuron.h, morphology): 
        """Create the cell model"""
        # Load morphology
        neuron.h.load_file(1, "morphology.hoc")
    if not hasattr(neuron.h, biophysics): 
        # Load biophysics
        neuron.h.load_file(1, "biophysics.hoc")
    if not hasattr(neuron.h, synapses):
        # load synapses
        neuron.h.load_file(1, os.path.join('synapses', 'synapses.hoc'))
    if not hasattr(neuron.h, templatename): 
        # Load main cell template
        neuron.h.load_file(1, "template.hoc")
    
    # create parameter dictionaries specific for each cell type (population)
    PSET.cellParameters[NRN] = dict(list(dict(
        morphology = glob(os.path.join('morphology', '*'))[0],
        templatefile = os.path.join(NRN, 'template.hoc'),
        templatename = templatename,
        templateargs=1 if add_synapses else 0,
        ).items()) + list(cellParams.items()))


# load synapse and connectivity data. mtype_map is the same for all cell types
if sys.version < '3':
    with open(os.path.join('synapses', 'mtype_map.tsv')) as f:
        mtype_map = np.loadtxt(f,
                               dtype={'names' : ('pre_mtype_id', 'pre_mtype'),
                                      'formats' : ('i4', '{}9'.format(stringType))},
                               converters = {1: lambda s: s.decode()})
else:
    with open(os.path.join('synapses', 'mtype_map.tsv'), encoding='us-ascii') as f:
        mtype_map = np.loadtxt(f, 
                               dtype={'names' : ('pre_mtype_id', 'pre_mtype'),
                                      'formats' : ('i4', '{}9'.format(stringType))},
                               converters = {1: lambda s: s.decode()})

os.chdir(PSET.CWD)


for name in PSET.populationParameters['m_type']:
    files = glob(os.path.join(PSET.CELLPATH, name+'*', 'synapses', 'synapses.tsv'))
    synapses_tsv[name] = np.array([], dtype=synapses_tsv_dtype)
    for f in files:
        synapses_tsv[name] = np.r_[synapses_tsv[name], np.loadtxt(f, dtype=synapses_tsv_dtype, skiprows = 1)]



# Open pathway anatomy and physiology factsheet files and read out info
pathways_anatomy = dict()
pathways_physiology = dict()

f = open(pathway_files[0], 'r')
j = json.load(f)
for pre in PSET.populationParameters['m_type']:
    for post in PSET.populationParameters['m_type']:
        key = '{}:{}'.format(pre, post)
        try:
            pathways_anatomy[key] = j[key]
        except KeyError:
            # fill in dummy data, no synapses will be created
            print('no pathway anatomy data for connection {}'.format(key))
            if sys.version < '3':
                pathways_anatomy[key] = {
                    'common_neighbor_bias': 0,
                    'connection_probability': 0,
                    'mean_number_of_synapse_per_connection': 0,
                    'number_of_convergent_neuron_mean': 0,
                    'number_of_convergent_neuron_std': 0,
                    'number_of_divergent_neuron_mean': 0,
                    'number_of_divergent_neuron_std': 0,
                    'number_of_synapse_per_connection_std': 0,
                    'total_synapse_count': 0,                    
                }            
            else:
                pathways_anatomy[key] = {
                    u'common_neighbor_bias': 0,
                    u'connection_probability': 0,
                    u'mean_number_of_synapse_per_connection': 0,
                    u'number_of_convergent_neuron_mean': 0,
                    u'number_of_convergent_neuron_std': 0,
                    u'number_of_divergent_neuron_mean': 0,
                    u'number_of_divergent_neuron_std': 0,
                    u'number_of_synapse_per_connection_std': 0,
                    u'total_synapse_count': 0,                    
                }
f.close()
j.clear()

f = open(pathway_files[1], 'r')
j = json.load(f)
for pre in PSET.populationParameters['m_type']:
    for post in PSET.populationParameters['m_type']:
        key = '{}:{}'.format(pre, post)
        try:
            pathways_physiology[key] = j[key]
        except KeyError:
            # fill in dummy data, no synapses will be created
            print('no pathway physiology data for connection {}'.format(key))
            if sys.version < '3':
                pathways_physiology[key] = {
                    'cv_psp_amplitude_mean': 3,
                    'cv_psp_amplitude_std': 0.95,
                    'd_mean': 360,
                    'd_std': 230,
                    'decay_mean': 9.8,
                    'decay_std': 6.7,
                    'epsp_mean': 1.6,
                    'epsp_std': 0.78,
                    'f_mean': 330,
                    'f_std': 240,
                    'failures_mean': 86,
                    'failures_std': 6.5,
                    'gsyn_mean': 0.3,
                    'gsyn_std': 0.11,
                    'latency_mean': 0.33,
                    'latency_std': 0.18,
                    'risetime_mean': 0.43,
                    'risetime_std': 0.47,
                    'space_clamp_correction_factor': 3.6,
                    'synapse_type': u'Excitatory, depressing',
                    'u_mean': 0.19,
                    'u_std': 0.23
                }            
            else:
                pathways_physiology[key] = {
                    u'cv_psp_amplitude_mean': 3,
                    u'cv_psp_amplitude_std': 0.95,
                    u'd_mean': 360,
                    u'd_std': 230,
                    u'decay_mean': 9.8,
                    u'decay_std': 6.7,
                    u'epsp_mean': 1.6,
                    u'epsp_std': 0.78,
                    u'f_mean': 330,
                    u'f_std': 240,
                    u'failures_mean': 86,
                    u'failures_std': 6.5,
                    u'gsyn_mean': 0.3,
                    u'gsyn_std': 0.11,
                    u'latency_mean': 0.33,
                    u'latency_std': 0.18,
                    u'risetime_mean': 0.43,
                    u'risetime_std': 0.47,
                    u'space_clamp_correction_factor': 3.6,
                    u'synapse_type': u'Excitatory, depressing',
                    u'u_mean': 0.19,
                    u'u_std': 0.23
                }
f.close()
j.clear()


# get out stats for synapses and connections, temporary
syn_param_stats = get_syn_params(PSET.populationParameters['m_type'],
                                 PSET.populationParameters['me_type'],
                                 pathways_physiology, mtype_map, synapses_tsv)

del synapses_tsv # not needed anymore. 


###########################################################################
# Set up main connection parameters used by Network class instance methods
############################################################################

# Main connection parameters between pre and post-synaptic populations organized as
# dictionary of parameter lists between pre and postsynaptic populations:
if PSET.fully_connected:
    connprob = [[1]*PSET.populationParameters.size]*PSET.populationParameters.size # fully connected network (no selfconnections)
else:
    connprob = get_params(PSET.populationParameters['m_type'], pathways_anatomy,
                          'connection_probability', 0.01*PSET.CONNPROBSCALING)  #unit conversion % -> fraction, increase weight  ewith diluted network

PSET.connParams = dict(
    # connection probabilities between populations    
    connprob = connprob,
    
    # synapse mechanisms
    syntypes = [[neuron.h.ProbAMPANMDA_EMS
                 if syn_param_stats['{}:{}'.format(pre, post)]['synapse_type'] >= 100 else
                 neuron.h.ProbGABAAB_EMS for post in PSET.populationParameters['m_type']] for pre in PSET.populationParameters['m_type']],
    
    
    # synapse time constants and reversal potentials.
    # Use the mean/global EPFL synapse model parameters
    # (for now) as some connections appear to be missing in pathway files.
    synparams = [[  dict(
                        Use = syn_param_stats['{}:{}'.format(pre, post)]['Use_mean'],
                        Dep = syn_param_stats['{}:{}'.format(pre, post)]['Dep_mean'],
                        Fac = syn_param_stats['{}:{}'.format(pre, post)]['Fac_mean'],
                        tau_r_AMPA = 0.2,
                        tau_d_AMPA = syn_param_stats['{}:{}'.format(pre, post)]['tau_d_mean'],
                        tau_r_NMDA = 0.29,
                        tau_d_NMDA = 43,
                        e=0,
                        mg=1,
                        u0=0,
                        synapseID=0,
                        verboseLevel=0,
                        NMDA_ratio=0.4 # this may take on several values in synconf.txt files, not acccounted for here
                    )
                if syn_param_stats['{}:{}'.format(pre, post)]['synapse_type'] >= 100 else
                    dict(
                        Use = syn_param_stats['{}:{}'.format(pre, post)]['Use_mean'],
                        Dep = syn_param_stats['{}:{}'.format(pre, post)]['Dep_mean'],
                        Fac = syn_param_stats['{}:{}'.format(pre, post)]['Fac_mean'],
                        tau_r_GABAA = 0.2, #from synapses.hoc: rng.lognormal(0.2, 0.1) (mean, variance)
                        tau_d_GABAA = syn_param_stats['{}:{}'.format(pre, post)]['tau_d_mean'],
                        tau_r_GABAB = 3.5,
                        tau_d_GABAB = 260.9,
                        e_GABAA = -80,
                        e_GABAB = -75.8354,
                        u0 = 0,
                        synapseID = 0,
                        verboseLevel = 0,
                        GABAB_ratio = 0.0, # this may take on several values, in synconf.txt files, not acccounted for here
                    )
                for post in PSET.populationParameters['m_type']] for pre in PSET.populationParameters['m_type']],

    # maximum conductances
    weightfuns = [[np.random.normal]*PSET.populationParameters.size]*PSET.populationParameters.size,
    weightargs = get_params(PSET.populationParameters['m_type'], pathways_physiology,
                            ['gsyn_mean', 'gsyn_std'], 1.),
    # Correct??? (very small PSPs otherwise).
    # Also, weights in unknown units loaded from synapses_tsv is different than the reported averaged gsyn. 
    
    # connection delays
    delayfuns = [[np.random.normal]*PSET.populationParameters.size]*PSET.populationParameters.size,
    delayargs = [[dict(
            loc=syn_param_stats['{}:{}'.format(pre, post)]['delay_mean'],
            scale=syn_param_stats['{}:{}'.format(pre, post)]['delay_std']
        ) for post in PSET.populationParameters['m_type']] for pre in PSET.populationParameters['m_type']],
    
    
    # numbers of synapses per connection
    multapsefuns = [[np.random.normal]*PSET.populationParameters.size]*PSET.populationParameters.size,
    multapseargs = get_params(PSET.populationParameters['m_type'], pathways_anatomy,
                              ['mean_number_of_synapse_per_connection',
                               'number_of_synapse_per_connection_std']),
    
    # parameters for finding random synapse locations using the method
    # LFPy.Cell.get_rand_idx_area_and_distribution_norm. The argument nidx is default to 1
    syn_pos_args = [[dict(section=syn_section,
                          z_min=-1E6,
                          z_max=1E6,
                          fun=[stats.norm]*PSET.layer_data.size,
                          funargs=[dict(loc=loc, scale=scale/2.) for loc, scale in PSET.layer_data[['center', 'thickness']]],
                          funweights=PSET.L_YXL_m_types[post_m_type][:, i]
                          ) for i, pre_m_type in enumerate(PSET.populationParameters['m_type'])] for post_m_type, syn_section in PSET.populationParameters[['m_type', 'syn_section']]],   
)

# save connection data
PSET.save_connections = True

# connection parameters for synapses activated by putative external population(s)
PSET.connParamsExtrinsic = dict(
    # synapse type
    syntype = 'ProbAMPANMDA_EMS',
    # synapse parameters (assumes parameters of excitatory population in the layer)
    synparams = [dict(
                        Use = syn_param_stats['{}:{}'.format(get_pre_m_type(post), post)]['Use_mean'],
                        Dep = syn_param_stats['{}:{}'.format(get_pre_m_type(post), post)]['Dep_mean'],
                        Fac = syn_param_stats['{}:{}'.format(get_pre_m_type(post), post)]['Fac_mean'],
                        tau_r_AMPA = 0.2,
                        tau_d_AMPA = syn_param_stats['{}:{}'.format(get_pre_m_type(post), post)]['tau_d_mean'],
                        tau_r_NMDA = 0.29,
                        tau_d_NMDA = 43,
                        e=0,
                        mg=1,
                        u0=0,
                        synapseID=0,
                        verboseLevel=0,
                        NMDA_ratio=0.4 # this may take on several values in synconf.txt files, not acccounted for here        
    ) for post in PSET.populationParameters['m_type']],

    # maximum conductances
    weightfuns = [np.random.normal]*PSET.populationParameters.size,
    weightargs = [get_params(np.array([m_type]), pathways_physiology,
                        ['gsyn_mean', 'gsyn_std'], 1.)[0][0] for m_type in PSET.populationParameters['m_type']],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
A very generic case where the same morphology on different file formats
is loaded in LFPy using a generic template specification, defined by
file LFPyCellTemplate.hoc

Execution:

    python example_LFPyCellTemplate.py
    
Copyright (C) 2017 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
"""
import LFPy
import neuron
import numpy as np
import matplotlib.pyplot as plt

#A plottin' function
def plotcell(cell, color='k'):
    for sec in cell.template.all:
        idx = cell.get_idx(sec.name())
        plt.plot(np.r_[cell.xstart[idx], cell.xend[idx][-1]],
                 np.r_[cell.zstart[idx], cell.zend[idx][-1]],
                 color=color)
    print(' ')

#delete cell instances from previous script executions,
neuron.h('forall delete_section()')

#create some cell instances, set the positions, plot the morphologies
cell1 = LFPy.TemplateCell(
    morphology='morphologies/markram/CNG version/C010398B-P2.CNG.swc',
    templatefile='LFPyCellTemplate.hoc',
    templatename='LFPyCellTemplate',
    templateargs=None)
cell1.set_pos(0)
plotcell(cell=cell1, color='r')


cell2 = LFPy.TemplateCell(
    morphology='morphologies/markram/Source-Version/C010398B-P2.asc',
    templatefile='LFPyCellTemplate.hoc',
    templatename='LFPyCellTemplate',
    templateargs=None)
cell2.set_pos(200)
plotcell(cell=cell2, color='g')


cell3 = LFPy.TemplateCell(
    morphology='morphologies/markram/hoc-version/C010398B-P2.hoc',
    templatefile='LFPyCellTemplate.hoc',
    templatename='LFPyCellTemplate',
    templateargs=None)
cell3.set_pos(400)
plotcell(cell=cell3, color='b')
plt.show()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''plotting script for figure 4 in manuscript preprint on output of
example_parallel_network.py

Copyright (C) 2018 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
'''
from __future__ import division
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
from matplotlib.collections import PolyCollection
from matplotlib.ticker import MaxNLocator
import os
import numpy as np
import h5py
from LFPy import NetworkCell
import example_parallel_network_plotting as plotting
from mpi4py import MPI

# set up MPI environment
COMM = MPI.COMM_WORLD
SIZE = COMM.Get_size()
RANK = COMM.Get_rank()


# # set default plotting parameters
plt.rcParams.update({
    'axes.xmargin': 0.0,
    'axes.ymargin': 0.0,
})
fontsize=14
titlesize=16
legendsize=12
plt.rcParams.update({
    'axes.xmargin': 0.0,
    'axes.ymargin': 0.0,
    'axes.labelsize': fontsize,
    'axes.titlesize': titlesize,
    'figure.titlesize': fontsize,
    'font.size': fontsize,
    'legend.fontsize': legendsize,
})

def plot_quantity_YXL(axes, PSET, quantity,
                      y=['p23', 'b23', 'nb23', 'p4', 'ss4(L23)', 'ss4(L4)',
                         'b4', 'nb4', 'p5(L23)', 'p5(L56)', 'b5', 'nb5',
                         'p6(L4)', 'p6(L56)', 'b6', 'nb6'],
                      label=r'$\mathcal{L}_{YXL}$', layers = ['L1', 'L2/3', 'L4', 'L5', 'L6'],
                      cmap=plt.get_cmap('inferno')):
                            
    '''make a bunch of image plots, each showing the spatial normalized
    connectivity of synapses'''
    if np.array(axes).ndim == 1:
        nrows = 1
        ncols = np.array(axes).size
    else:
        (nrows, ncols) = np.array(axes).shape
    
    
    #assess vlims
    vmin = 0
    vmax = 0
    for yi in y:
        if quantity[yi].max() > vmax:
            vmax = quantity[yi].max()
    
    for i, yi in enumerate(y):
        ax = np.array(axes).flatten()[i]

        masked_array = np.ma.array(quantity[yi], mask=quantity[yi]==0)
        
        im = ax.pcolormesh(masked_array,
                            vmin=vmin, vmax=vmax,
                            cmap=cmap,
                            )
        ax.invert_yaxis()

        ax.axis(ax.axis('tight'))
        ax.set_xticks(np.arange(len(y))+0.5)
        ax.set_yticks(np.arange(len(layers))+0.5)
        
        if i % ncols == 0:
            ax.set_yticklabels(layers, )
            ax.set_ylabel('$L$', labelpad=0.)
        else:
            ax.set_yticklabels([])
        if i < ncols:
            ax.set_xlabel(r'$X$', labelpad=-1)
            ax.set_xticklabels(y, rotation=90)
        else:
            ax.set_xticklabels([])
        
        ax.set_title(r'$Y=${}'.format(yi))
    
        #colorbar
        if (i // ncols==0) and (i % ncols) == ncols-1:
            rect = np.array(ax.get_position().bounds)
            rect[0] += rect[2] + 0.0025
            rect[2] = 0.005
            cax = fig.add_axes(rect)
            cbar = plt.colorbar(im, cax=cax)
            cbar.set_label(label, labelpad=0)

def plot_multapseargs(ax, PSET, cmap=plt.get_cmap('inferno'),
                      data='multapseargs', cbarlabel=r'$\overline{n}_\mathrm{syn}$',
                      key='loc'):

    '''make an imshow of connectivity parameters'''
    item = PSET.connParams[data]
    array = [[i[key] for i in j]  for j in item]
    
    masked_array = np.ma.array(array, mask=np.array(array)==0.)
    cmap.set_bad('k', 0.5)
    im = ax.pcolormesh(masked_array, cmap=cmap, vmin=0, )
    ax.axis(ax.axis('tight'))
    ax.invert_yaxis()
    ax.xaxis.set_ticks_position('top')
    ax.set_xticks(np.arange(PSET.populationParameters.size)+0.5)
    ax.set_yticks(np.arange(PSET.populationParameters.size)+0.5)
    ax.set_xticklabels(PSET.populationParameters['m_type'], rotation=270)
    ax.set_yticklabels(PSET.populationParameters['m_type'], )
    ax.xaxis.set_label_position('top')
    ax.set_xlabel(r'$Y$', labelpad=0)
    ax.set_ylabel(r'$X$', labelpad=0, rotation=0)

    rect = np.array(ax.get_position().bounds)
    rect[0] += rect[2] + 0.0025
    rect[2] = 0.005
    fig = plt.gcf()
    cax = fig.add_axes(rect)

    cbar = plt.colorbar(im, cax=cax)
    cbar.set_label(cbarlabel, labelpad=0)


if __name__ == '__main__':
    # get simulation parameters
    from example_parallel_network_parameters import PSET

    # cell type colors 
    colors = [plt.get_cmap('Set1', PSET.populationParameters.size)(i) for i in range(PSET.populationParameters.size)]

    # Set up figure and subplots 
    fig = plt.figure(figsize=(16, 10))
    fig.subplots_adjust(left=0.075, right=0.95, hspace=0.2, wspace=0.7,
                        bottom=0.05, top=0.95)
    gs = GridSpec(16, 40, hspace=0.2, wspace=0.5)
    alphabet = 'ABCDEFG'
    dz = 50 #spatial resolution of histograms
    
    # PANEL A. Morphology overview
    ax = fig.add_subplot(gs[:12, :12])
    plotting.remove_axis_junk(ax, lines=['top', 'bottom', 'right'])
    n_segs, areas = plotting.plot_m_types(ax, PSET, colors, spacing=300.)
    ax.set_title('')
    ax.set_xticks([])
    ax.set_xticklabels([])
    ax.set_ylabel('layer')
    ax.text(-0.05, 1.05, alphabet[0],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)
    axis = ax.axis()


    ax = fig.add_subplot(gs[12:, :12])
    ax.axis('off')
    ax.set_xlim(left=axis[0], right=axis[1])

    # # table data with population sizes etc.
    h_spacing=300.
    v_spacing=100.
    keys =   ['m_type',              'm_type',               'e_type',             'me_type',       'n_seg',                           'POP_SIZE',                 'F_y',                 'extrinsic_input_density', 'extrinsic_input_frequency',
              'pop_args[loc]',                          'pop_args[scale]']
    labels = ['population ($X/Y$):', 'morphology type (m):', 'electric type (e):', 'cell model #:', 'segment count $n_\mathrm{seg}$:', 'population count $N_X$:', r'Occurrence $F_X$:',  r'$n_\mathrm{ext}$:',      r'$\nu_\mathrm{ext}$ (s$^{-1}$):',
              r'$\overline{z}_X^\mathrm{soma}$ ($\mu$m):', r'$\sigma_{\overline{z},X}^\mathrm{soma}$ ($\mu$m)']
    # PSET.populationParameters
    z = -PSET.layer_data['thickness'].sum()
    ax.set_ylim(top=z-v_spacing, bottom=z-len(keys)*v_spacing+v_spacing)
    for i, (label, key) in enumerate(zip(labels, keys)):
        ax.text(axis[0], z-(i+1)*v_spacing, label, ha='right', va='top')
        for j, popData in enumerate(PSET.populationParameters):
            if key == 'pop_args[loc]':
                ax.text(j*h_spacing, z-(i+1)*v_spacing, int(popData['pop_args']['loc']), ha='center', va='top')
            elif key == 'pop_args[scale]':
                ax.text(j*h_spacing, z-(i+1)*v_spacing, int(popData['pop_args']['scale']), ha='center', va='top')
            elif key == 'me_type':
                ax.text(j*h_spacing, z-(i+1)*v_spacing, popData[key].lstrip(popData['m_type'] + '_' + popData['e_type']), ha='center', va='top')
            elif key == 'n_seg':
                ax.text(j*h_spacing, z-(i+1)*v_spacing, n_segs[j], ha='center', va='top')
            elif key == 'F_y':
                ax.text(j*h_spacing, z-(i+1)*v_spacing, '{:.2f}'.format(popData['POP_SIZE']/PSET.populationParameters['POP_SIZE'].sum()), ha='center', va='top')
            elif key == 'extrinsic_input_density':
                ax.text(j*h_spacing, z-(i+1)*v_spacing, '{:.0f}'.format(int(areas[j]*popData[key])), ha='center', va='top')
            else:
                ax.text(j*h_spacing, z-(i+1)*v_spacing, popData[key], ha='center', va='top')
    
    
    # PANEL B. Connection probability
    ax = fig.add_subplot(gs[:3, 14:17])
    plotting.plot_connectivity(ax, PSET, data='connprob', cbarlabel='')
    ax.xaxis.set_ticks_position('bottom')
    ax.xaxis.set_label_position('bottom')
    ax.set_title(r'$C_{YX}$')
    ax.text(-0.1, 1.15, alphabet[1],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)
    
    # PANEL C. Mean number of synapses per connection
    ax = fig.add_subplot(gs[:3, 19:22])
    plot_multapseargs(ax, PSET, data='multapseargs', cbarlabel='', key='loc')
    ax.set_yticklabels([])
    ax.set_ylabel('')
    ax.xaxis.set_ticks_position('bottom')
    ax.xaxis.set_label_position('bottom')
    ax.set_title(r'$\overline{n}_\mathrm{syn}$')
    ax.text(-0.1, 1.15, alphabet[2],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)

    
    # PANEL D. Layer specificites of connections
    axes = [fig.add_subplot(gs[:3, i*4+4:i*4+8]) for i in range(5, 9)]
    plot_quantity_YXL(axes=axes, PSET=PSET,
                      quantity=PSET.L_YXL_m_types,
                      y=PSET.populationParameters['m_type'],
                      layers = PSET.layer_data['layer'],
                      label=r'$\mathcal{L}_{YXL}$')
    for ax in axes:
        ax.set_ylabel('')
    axes[0].text(-0.075, 1.15, alphabet[3],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=axes[0].transAxes)

    
    # PANEL E. Population illustration
    ax = fig.add_subplot(gs[6:, 13:20])
    # plot electrode contact points
    ax.plot(PSET.electrodeParams['x'], PSET.electrodeParams['z'], 'ko',
            markersize=5, clip_on=False)
    # plot ECoG electrode
    ax.plot([-PSET.ecogParameters['r'], PSET.ecogParameters['r']], [PSET.ecogParameters['z'][0]]*2, 'gray', lw=5, zorder=-1, clip_on=False)
    
    plotting.remove_axis_junk(ax)

    # draw the first NCELLS cells in each population
    NCELLS = 20
    CWD=PSET.CWD
    CELLPATH=PSET.CELLPATH
    # cell positions and rotations file:
    f = h5py.File(os.path.join(PSET.OUTPUTPATH,
                               'cell_positions_and_rotations.h5'), 'r')
    
    for i, data in enumerate(PSET.populationParameters):
        NRN = data["me_type"]
        os.chdir(os.path.join(CWD, CELLPATH, NRN))
        for j in range(NCELLS):
            try:                
                cell = NetworkCell(**PSET.cellParameters[NRN])
                cell.set_pos(x=f[NRN]['x'][j], y=f[NRN]['y'][j], z=f[NRN]['z'][j])
                cell.set_rotation(x=f[NRN]['x_rot'][j], y=f[NRN]['y_rot'][j], z=f[NRN]['z_rot'][j])
        
                zips = []
                for x, z in cell.get_idx_polygons(projection=('x', 'z')):
                    zips.append(list(zip(x, z)))
        
                polycol = PolyCollection(zips,
                                         edgecolors=colors[i],
                                         linewidths=0.05,
                                         facecolors=colors[i],
                                         label=NRN,
                                         zorder=f[NRN]['y'][j],
                                         )
                ax.add_collection(polycol)
                neuron.h('forall delete_section()')
            except IndexError:
                pass # NCELLS > cell count in population, plotting all there is. 
        os.chdir(CWD)

    ax.axis(ax.axis('equal'))
    ax.set_ylim(-PSET.layer_data['thickness'].sum(), 0)
    # draw lines showing the layer boundaries
    ax.hlines(np.r_[0., -PSET.layer_data['thickness'].cumsum()], -536.09990331936808, 536.09990331936808, 'k', lw=0.5)
    ax.set_yticks(PSET.layer_data['center'])
    ax.set_yticklabels(PSET.layer_data['layer'])
    ax.set_ylabel('layer')
    ax.set_xticks([-data['pop_args']['radius'], 0, data['pop_args']['radius']])
    ax.set_xlabel('$x$ ($\mu$m)')
    

    ax.text(-0.05, 1.05, alphabet[4],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)

    
    # PANEL F. Population densities across depth 
    ax = fig.add_subplot(gs[6:, 20:24])
    plotting.remove_axis_junk(ax)
    ax.text(-0.075, 1.05, alphabet[5],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)
    
    # open file for reading
    f = h5py.File(os.path.join(PSET.OUTPUTPATH,
                               'cell_positions_and_rotations.h5'), 'r')
    # spatial bins across depth
    bins = np.arange(0, -PSET.layer_data['thickness'].sum(), -50)[::-1]
    for color, post, key in zip(colors, PSET.populationParameters['m_type'], PSET.populationParameters['me_type']):
        ax.hist(f[key]['z'], bins=bins, color=color, alpha=1, orientation='horizontal', histtype='step', label=r'{}'.format(post), clip_on=False)
    ax.set_xlabel('count')
    axis = ax.axis('tight')
    ax.axis(axis)
    ax.set_ylim(-PSET.layer_data['thickness'].sum(), 0)
    # draw lines showing the layer boundaries
    ax.hlines(np.r_[0., -PSET.layer_data['thickness'].cumsum()], axis[0], axis[1], 'k', lw=0.5)
    ax.set_yticks(PSET.layer_data['center'])
    ax.set_yticklabels([])
    ax.legend(loc=8)
    ax.xaxis.set_major_locator(MaxNLocator(3))
    

    # PANEL G. Resulting densities of synapses across depth onto each cell type
    axes = [fig.add_subplot(gs[6:, i*4+4:i*4+8]) for i in range(5, 9)]
    axes[0].text(-0.075, 1.05, alphabet[6],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=axes[0].transAxes)
    
    # spatial bins across depth
    bins = np.arange(0, -PSET.layer_data['thickness'].sum(), -50)[::-1]

    # file output
    f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'synapse_positions.h5'))
    for i, (m_post, post) in enumerate(zip(PSET.populationParameters['m_type'], PSET.populationParameters['me_type'])):
        ax = axes[i]
        plotting.remove_axis_junk(ax)
        ax.set_xlabel('count ($10^3$)')
        ax.set_yticklabels([])
        ax.set_title(r'$Y=${}'.format(m_post))
        for color, m_pre, pre in zip(colors, PSET.populationParameters['m_type'], PSET.populationParameters['me_type']):
            key = '{}:{}'.format(pre, post)
            ax.hist(f[key]['z'], bins=bins, color=color, alpha=1, orientation='horizontal', histtype='step', label=r'{}'.format(m_pre), clip_on=False, weights=1E-3*np.ones(f[key]['z'].size))
        axis = ax.axis('tight')
        ax.axis(axis)
        ax.set_ylim(-PSET.layer_data['thickness'].sum(), 0)
        # draw lines showing the layer boundaries
        ax.hlines(np.r_[0., -PSET.layer_data['thickness'].cumsum()], axis[0], axis[1], 'k', lw=0.5)
        ax.set_yticks(PSET.layer_data['center'])
        ax.xaxis.set_major_locator(MaxNLocator(2))
        

                


    fig.savefig(os.path.join(PSET.OUTPUTPATH, 'figure_4.pdf'), bbox_inches='tight')

    plt.show()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Demonstrate usage of LFPy.NetworkPopulation with a ball-and-stick type
morphology with active HH channels inserted in the soma and passive-leak
channels distributed throughout the apical stick. The corresponding morphology
and template-specification is in the files BallAndStick.hoc and
BallAndStickTemplate.hoc.

Execution:

    python example_network_population.py
    
Copyright (C) 2017 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""
# import modules
from mpi4py.MPI import COMM_WORLD as COMM
from LFPy import NetworkPopulation, NetworkCell
# class NetworkCell parameters:
cellParameters = dict(
    morphology='BallAndStick.hoc',
    templatefile='BallAndStickTemplate.hoc',
    templatename='BallAndStickTemplate',
    templateargs=None,
    delete_sections=False,
)
# class NetworkPopulation parameters:
populationParameters = dict(
    Cell=NetworkCell,
    cell_args=cellParameters,
    pop_args=dict(
        radius=100.,
        loc=0.,
        scale=20.),
    rotation_args=dict(x=0., y=0.),
)
# create population:
population = NetworkPopulation(
    first_gid=0, name='E',
    **populationParameters
)
# print out some info:
for cell in population.cells:
    print('RANK {}; pop {}; gid {}; cell {}'.format(
        COMM.Get_rank(), population.name,
        cell.gid, cell))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Test implementation using cell models of the Blue Brain Project with LFPy.
The example assumes that the complete set of cell models available from
https://bbpnmc.epfl.ch/nmc-portal/downloads is unzipped in this folder.

Execution:

    python example_EPFL_neurons.py

Copyright (C) 2017 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
"""

import os
import posixpath
import sys
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
from matplotlib.collections import PolyCollection, LineCollection
from glob import glob
import numpy as np
from warnings import warn
import scipy.signal as ss
import neuron
import LFPy
from mpi4py import MPI


plt.rcParams.update({
    'axes.labelsize': 8,
    'axes.titlesize': 8,
    'font.size': 8,
    'ytick.labelsize': 8,
    'xtick.labelsize': 8,
    })

COMM = MPI.COMM_WORLD
SIZE = COMM.Get_size()
RANK = COMM.Get_rank()


# working dir
CWD = os.getcwd()
NMODL = 'hoc_combos_syn.1_0_10.allmods'

# load some required neuron-interface files
neuron.h.load_file("stdrun.hoc")
neuron.h.load_file("import3d.hoc")

# load only some layer 5 pyramidal cell types
neurons = glob(os.path.join('hoc_combos_syn.1_0_10.allzips', 'L5_TTPC*'))[:1]
neurons += glob(os.path.join('hoc_combos_syn.1_0_10.allzips', 'L5_MC*'))[:1]
neurons += glob(os.path.join('hoc_combos_syn.1_0_10.allzips', 'L5_LBC*'))[:1]
neurons += glob(os.path.join('hoc_combos_syn.1_0_10.allzips', 'L5_NBC*'))[:1]

# flag for cell template file to switch on (inactive) synapses
add_synapses = False

# attempt to set up a folder with all unique mechanism mod files, compile, and
# load them all
if RANK == 0:
    if not os.path.isdir(NMODL):
        os.mkdir(NMODL)
    for NRN in neurons:
        for nmodl in glob(os.path.join(NRN, 'mechanisms', '*.mod')):
            while not os.path.isfile(os.path.join(NMODL,
                                                  os.path.split(nmodl)[-1])):
                if "win32" in sys.platform:
                    os.system("copy {} {}".format(nmodl, NMODL))
                else:
                    os.system('cp {} {}'.format(nmodl,
                                                os.path.join(NMODL, '.')))
    os.chdir(NMODL)
    if "win32" in sys.platform:
        warn("no autompile of NMODL (.mod) files on Windows. " +
             "Run mknrndll from NEURON bash in the folder %s" % NMODL +
             "and rerun example script")
    else:
        os.system('nrnivmodl')
    os.chdir(CWD)
COMM.Barrier()
if "win32" in sys.platform:
    if NMODL not in neuron.nrn_dll_loaded:
        neuron.h.nrn_load_dll(NMODL+"/nrnmech.dll")
    neuron.nrn_dll_loaded.append(NMODL)
else:
    neuron.load_mechanisms(NMODL)

os.chdir(CWD)

FIGS = 'hoc_combos_syn.1_0_10.allfigures'
if not os.path.isdir(FIGS):
    os.mkdir(FIGS)


# load the LFPy SinSyn mechanism for stimulus
if "win32" in sys.platform:
    pth = os.path.join(LFPy.__path__[0], "test").replace(os.sep, posixpath.sep)
    if pth not in neuron.nrn_dll_loaded:
        neuron.h.nrn_load_dll(pth + "/nrnmech.dll")
    neuron.nrn_dll_loaded.append(pth)
else:
    neuron.load_mechanisms(os.path.join(LFPy.__path__[0], "test"))


def posixpth(pth):
    """
    Replace Windows path separators with posix style separators
    """
    return pth.replace(os.sep, posixpath.sep)


def get_templatename(f):
    '''
    Assess from hoc file the templatename being specified within

    Arguments
    ---------
    f : file, mode 'r'

    Returns
    -------
    templatename : str

    '''
    for line in f.readlines():
        if 'begintemplate' in line.split():
            templatename = line.split()[-1]
            print('template {} found!'.format(templatename))
            continue

    return templatename


# PARAMETERS

# sim duration
tstop = 1000.
dt = 2**-6

PointProcParams = {
    'idx': 0,
    'pptype': 'SinSyn',
    'delay': 200.,
    'dur': tstop - 300.,
    'pkamp': 0.5,
    'freq': 0.,
    'phase': np.pi/2,
    'bias': 0.,
    'record_current': False
}

# spike sampling
threshold = -20  # spike threshold (mV)
samplelength = int(2. / dt)

# filter settings for extracellular traces
b, a = ss.butter(N=3, Wn=(300*dt*2/1000, 5000*dt*2/1000), btype='bandpass')
apply_filter = True

# communication buffer where all simulation output will be gathered on RANK 0
COMM_DICT = {}

COUNTER = 0
for i, NRN in enumerate(neurons):
    os.chdir(NRN)

    # get the template name
    f = open("template.hoc", 'r')
    templatename = get_templatename(f)
    f.close()

    # get biophys template name
    f = open("biophysics.hoc", 'r')
    biophysics = get_templatename(f)
    f.close()

    # get morphology template name
    f = open("morphology.hoc", 'r')
    morphology = get_templatename(f)
    f.close()

    # get synapses template name
    f = open(posixpth(os.path.join("synapses", "synapses.hoc")), 'r')
    synapses = get_templatename(f)
    f.close()

    print('Loading constants')
    neuron.h.load_file('constants.hoc')

    if not hasattr(neuron.h, morphology):
        """Create the cell model"""
        # Load morphology
        neuron.h.load_file(1, "morphology.hoc")
    if not hasattr(neuron.h, biophysics):
        # Load biophysics
        neuron.h.load_file(1, "biophysics.hoc")
    if not hasattr(neuron.h, synapses):
        # load synapses
        neuron.h.load_file(1, posixpth(os.path.join('synapses', 'synapses.hoc')
                                       ))
    if not hasattr(neuron.h, templatename):
        # Load main cell template
        neuron.h.load_file(1, "template.hoc")

    for morphologyfile in glob(os.path.join('morphology', '*')):
        if COUNTER % SIZE == RANK:
            # Instantiate the cell(s) using LFPy
            cell = LFPy.TemplateCell(morphology=morphologyfile,
                                     templatefile=posixpth(os.path.join(NRN,
                                                           'template.hoc')),
                                     templatename=templatename,
                                     templateargs=1 if add_synapses else 0,
                                     tstop=tstop,
                                     dt=dt,
                                     nsegs_method=None)

            # set view as in most other examples
            cell.set_rotation(x=np.pi/2)

            pointProcess = LFPy.StimIntElectrode(cell, **PointProcParams)

            electrode = LFPy.RecExtElectrode(x=np.array([-40, 40., 0, 0]),
                                             y=np.array([0, 0, -40, 40]),
                                             z=np.zeros(4),
                                             sigma=0.3, r=5, n=50,
                                             N=np.array([[1, 0, 0],
                                                         [1, 0, 0],
                                                         [1, 0, 0],
                                                         [1, 0, 0]]),
                                             method='soma_as_point')
            # run simulation
            cell.simulate(electrode=electrode)

            # compute LFP
            if apply_filter:
                LFP = ss.filtfilt(b, a, electrode.LFP, axis=-1)

            # detect action potentials from intracellular trace
            AP_train = np.zeros(cell.somav.size, dtype=int)
            crossings = ((cell.somav[:-1] < threshold) &
                         (cell.somav[1:] >= threshold))
            spike_inds = np.where(crossings)[0]
            # sampled spike waveforms for each event
            spw = np.zeros((crossings.sum()*LFP.shape[0], 2*samplelength))
            tspw = np.arange(-samplelength, samplelength)*dt
            # set spike time where voltage gradient is largest
            n = 0  # counter
            for j, i in enumerate(spike_inds):
                inds = np.arange(i - samplelength, i + samplelength)
                w = cell.somav[inds]
                k = inds[:-1][np.diff(w) == np.diff(w).max()][0]
                AP_train[k] = 1
                # sample spike waveform
                for l in LFP:
                    spw[n, ] = l[np.arange(k - samplelength, k + samplelength)]
                    n += 1

            # fill in sampled spike waveforms and times of spikes in comm_dict
            COMM_DICT.update({
                os.path.split(NRN)[-1] + '_' +
                os.path.split(morphologyfile)[-1].strip('.asc'): dict(
                    spw=spw,
                )
            })

            # plot
            gs = GridSpec(2, 3)
            fig = plt.figure(figsize=(10, 8))
            fig.suptitle(NRN + '\n' +
                         os.path.split(morphologyfile)[-1].strip('.asc'))

            # morphology
            zips = []
            for x, z in cell.get_idx_polygons(projection=('x', 'z')):
                zips.append(list(zip(x, z)))
            polycol = PolyCollection(zips,
                                     edgecolors='none',
                                     facecolors='k',
                                     rasterized=True)
            ax = fig.add_subplot(gs[:, 0])
            ax.add_collection(polycol)
            ax.plot(electrode.x, electrode.z, 'ro')
            ax.axis(ax.axis('equal'))
            ax.set_title('morphology')
            ax.set_xlabel('(um)', labelpad=0)
            ax.set_ylabel('(um)', labelpad=0)

            # soma potential and spikes
            ax = fig.add_subplot(gs[0, 1])
            ax.plot(cell.tvec, cell.somav, rasterized=True)
            ax.plot(cell.tvec, AP_train*20 + 50)
            ax.axis(ax.axis('tight'))
            ax.set_title('soma voltage, spikes')
            ax.set_ylabel('(mV)', labelpad=0)

            # extracellular potential
            ax = fig.add_subplot(gs[1, 1])
            for l in LFP:
                ax.plot(cell.tvec, l, rasterized=True)
            ax.axis(ax.axis('tight'))
            ax.set_title('extracellular potential')
            ax.set_xlabel('(ms)', labelpad=0)
            ax.set_ylabel('(mV)', labelpad=0)

            # spike waveform
            ax = fig.add_subplot(gs[0, 2])
            n = electrode.x.size
            for j in range(n):
                zips = []
                for x in spw[j::n, ]:
                    zips.append(list(zip(tspw, x)))
                linecol = LineCollection(zips,
                                         linewidths=0.5,
                                         colors=plt.cm.Spectral(int(255.*j/n)),
                                         rasterized=True)
                ax.add_collection(linecol)
            ax.axis(ax.axis('tight'))
            ax.set_title('spike waveforms')
            ax.set_ylabel('(mV)', labelpad=0)

            # spike width vs. p2p amplitude
            ax = fig.add_subplot(gs[1, 2])
            w = []
            p2p = []
            for x in spw:
                j = x == x.min()
                i = x == x[np.where(j)[0][0]:].max()
                w += [(tspw[i] - tspw[j])[0]]
                p2p += [(x[i] - x[j])[0]]
            ax.plot(w, p2p, 'o', lw=0.1, markersize=5, mec='none')
            ax.set_title('spike peak-2-peak time and amplitude')
            ax.set_xlabel('(ms)', labelpad=0)
            ax.set_ylabel('(mV)', labelpad=0)

            fig.savefig(os.path.join(CWD, FIGS, os.path.split(NRN)[-1] + '_' +
                        os.path.split(morphologyfile
                                      )[-1].replace('.asc', '.pdf')),
                        dpi=200)
            plt.close(fig)

        COUNTER += 1
        os.chdir(CWD)


COMM.Barrier()

# gather sim output
if SIZE > 1:
    if RANK == 0:
        for i in range(1, SIZE):
            COMM_DICT.update(COMM.recv(source=i, tag=123))
            print('received from RANK {} on RANK {}'.format(i, RANK))
    else:
        print('sent from RANK {}'.format(RANK))
        COMM.send(COMM_DICT, dest=0, tag=123)
else:
    pass
COMM.Barrier()

# project data
if RANK == 0:
    fig = plt.figure(figsize=(10, 8))
    fig.suptitle('spike peak-2-peak time and amplitude')
    n = electrode.x.size
    for k in range(n):
        ax = fig.add_subplot(n, 2, k*2+1)
        for key, val in COMM_DICT.items():
            spw = val['spw'][k::n, ]
            w = []
            p2p = []
            for x in spw:
                j = x == x.min()
                i = x == x[np.where(j)[0][0]:].max()
                w += [(tspw[i] - tspw[j])[0]]
                p2p += [(x[i] - x[j])[0]]
            if 'MC' in key:
                marker = 'x'
            elif 'NBC' in key:
                marker = '+'
            elif 'LBC' in key:
                marker = 'd'
            elif 'TTPC' in key:
                marker = '^'
            ax.plot(w, p2p, marker, lw=0.1,
                    markersize=5, mec='none', label=key, alpha=0.25)
        ax.set_xlabel('(ms)', labelpad=0)
        ax.set_ylabel('(mV)', labelpad=0)
        if k == 0:
            ax.legend(loc='upper left', bbox_to_anchor=(1, 1),
                      frameon=False, fontsize=7)
    fig.savefig(os.path.join(CWD, FIGS, 'P2P_time_amplitude.pdf'))
    print("wrote {}".format(os.path.join(CWD, FIGS, 'P2P_time_amplitude.pdf')))
    plt.close(fig)
else:
    pass
COMM.Barrier()



#!/usr/bin/env python
"""Copyright (C) 2012 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""


import numpy as np
import scipy.stats
import warnings

def get_activation_times_from_distribution(n, tstart=0., tstop=1.E6,
                                          distribution=scipy.stats.expon,
                                          rvs_args=dict(loc=0, scale=1),
                                          maxiter=1E6):
    """
    Construct a length n list of ndarrays containing continously increasing
    random numbers on the interval [tstart, tstop], with intervals drawn from
    a chosen continuous random variable distribution subclassed from
    scipy.stats.rv_continous, e.g., scipy.stats.expon or scipy.stats.gamma.
    
    The most likely initial first entry is
    ``tstart + method.rvs(size=inf, **rvs_args).mean()``
    
    Parameters
    ----------
    n : int
        number of ndarrays in list
    tstart : float
        minimum allowed value in ndarrays
    tstop : float
        maximum allowed value in ndarrays
    distribution : object
        subclass of scipy.stats.rv_continous. Distributions
        producing negative values should be avoided if continously increasing
        values should be obtained, i.e., the probability density function
        ``(distribution.pdf(**rvs_args))`` should be ``0`` for ``x < 0``, but this is not
        explicitly tested for.
    rvs_args : dict
        parameters for method.rvs method. If "size" is in dict, then tstop will
        be ignored, and each ndarray in output list will be
        ``distribution.rvs(**rvs_args).cumsum() + tstart``. If size is not given in dict,
        then values up to tstop will be included
    maxiter : int
        maximum number of iterations
        
        
    Returns
    -------
    list of ndarrays
        length n list of arrays containing data
    
    Raises
    ------
    AssertionError
        if distribution does not have the 'rvs' attribute
    StopIteration
        if number of while-loop iterations reaches maxiter
    
    
    Examples
    --------
    Create n sets of activation times with intervals drawn from the exponential
    distribution, with rate expectation lambda 10 s^-1 (thus
    scale=1000 / lambda). Here we assume output in units of ms
    
    >>> from LFPy.inputgenerators import get_activation_times_from_distribution
    >>> import scipy.stats as st
    >>> import matplotlib.pyplot as plt
    >>> times = get_activation_times_from_distribution(n=10, tstart=0., tstop=1000.,
    >>>                                             distribution=st.expon,
    >>>                                             rvs_args=dict(loc=0.,
    >>>                                                           scale=100.))
    """
    try:
        assert hasattr(distribution, 'rvs')
    except AssertionError:
        raise AssertionError('distribution={} must have the attribute "rvs"'.format(distribution))
    
    times = []
    if 'size' in rvs_args.keys():
        for i in range(n):
            times += [distribution.rvs(**rvs_args).cumsum() + tstart]
    else:
        for i in range(n):
            values = distribution.rvs(size=1000, **rvs_args).cumsum() + tstart
            iter = 0
            while values[-1] < tstop and iter < maxiter:
                values = np.r_[values, distribution.rvs(size=1000, **rvs_args).cumsum() + values[-1]]
                iter += 1
            
            if iter == maxiter:
                raise StopIteration('maximum number of iterations reach. Con')            
            
            times += [values[values < tstop]]
    
    return times




#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''plotting script for figure 6 in manuscript preprint on output of
example_parallel_network.py

Copyright (C) 2018 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
'''
from __future__ import division
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
import os
import numpy as np
import h5py
import example_parallel_network_plotting as plotting
from mpi4py import MPI

# set up MPI environment
COMM = MPI.COMM_WORLD
SIZE = COMM.Get_size()
RANK = COMM.Get_rank()

fontsize=14
titlesize=16
legendsize=12
plt.rcParams.update({
    'axes.xmargin': 0.0,
    'axes.ymargin': 0.0,
    'axes.labelsize': fontsize,
    'axes.titlesize': titlesize,
    'figure.titlesize': fontsize,
    'font.size': fontsize,
    'legend.fontsize': legendsize,
})

if __name__ == '__main__':
    # get simulation parameters
    from example_parallel_network_parameters import PSET

    # cell type colors 
    colors = [plt.get_cmap('Set1', PSET.populationParameters.size)(i) for i in range(PSET.populationParameters.size)]

    # time shown
    T = (PSET.TRANSIENT, PSET.TRANSIENT+1000.)

    # Set up figure and subplots 
    fig = plt.figure(figsize=(16, 12))
    gs = GridSpec(15, 5, left=0.075, right=0.975,
                  top=0.95, bottom=0.05, wspace=0.3, hspace=0.2)
    alphabet = 'ABCDEFGHIJKLMNOPQ'


    for j, (m_type, me_type) in enumerate(zip(PSET.populationParameters['m_type'], PSET.populationParameters['me_type'])):
        ax = fig.add_subplot(gs[:8, j])
        f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
        for data, title, color in zip(
                               [f['SUMMED_OUTPUT'][()][me_type]],
                               [m_type],
                               ['k']):
            ax.set_title(title)
            vlimround = plotting.draw_lineplot(ax=ax, data=plotting.decimate(data, q=PSET.decimate_q),
                                      dt=PSET.dt*PSET.decimate_q,
                                      T=T, color=color,
                                      scalebarbasis='log10')
        
        if j > 0:
            ax.set_yticklabels([])
            ax.set_ylabel('')
        
        ax.text(-0.1, 1.05, alphabet[j],
            horizontalalignment='center',
            verticalalignment='center',
            fontsize=16, fontweight='demibold',
            transform=ax.transAxes)
        
        

        p = f['CURRENT_DIPOLE_MOMENT'][me_type] * 1E-3 # nA um -> 1E-3 nA m unit conversion
        for i, (u, ls, lw, ylbl) in enumerate(zip(['x', 'y', 'z'], ['-', '-', '-'], [1, 1, 1],
            [r'$\mathbf{p \cdot \hat{x}}$' + '\n' + r'($10^{-3}$ nA m)',
             r'$\mathbf{p \cdot \hat{y}}$' + '\n' + r'($10^{-3}$ nA m)',
             r'$\mathbf{p \cdot \hat{z}}$' + '\n' + r'($10^{-3}$ nA m)'])):
            ax = fig.add_subplot(gs[9+i*2:11+i*2, j])
            if i == 0:
                ax.text(-0.1, 1.2, alphabet[j+5],
                    horizontalalignment='center',
                    verticalalignment='center',
                    fontsize=16, fontweight='demibold',
                    transform=ax.transAxes)

            plotting.remove_axis_junk(ax)
            x = plotting.decimate(p[:, i], q=PSET.decimate_q)
            t = np.arange(x.size)*PSET.dt*PSET.decimate_q
            inds = (t >= T[0]) & (t <= T[1])
            ax.plot(t[inds], x[inds], ls=ls, lw=lw, 
                    color='k')
            if i != 2:
                ax.set_xticklabels([])
            if j == 0:
                ax.set_ylabel(ylbl, labelpad=0)

        f.close()
        
        ax.set_xlabel('time (ms)', labelpad=0)
    
        

    ax = fig.add_subplot(gs[:8, 4])
    ax.set_title('signal variance')
    y = PSET.electrodeParams['z']
    tind = int(PSET.TRANSIENT / PSET.dt)        
    f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
    
    for m_type, me_type, color in zip(list(PSET.populationParameters['m_type'])+['summed'], list(PSET.populationParameters['me_type'])+['imem'], colors+['k']):
        data = f['SUMMED_OUTPUT'][()][me_type]
        ax.semilogx(data[:, tind:].var(axis=1), y, lw=2, label=m_type, color=color)
    f.close()
    ax.set_yticks(y)
    ax.set_yticklabels([])
    ax.set_ylabel('')
    plotting.remove_axis_junk(ax)
    ax.axis(ax.axis('tight'))
    ax.legend(loc='best')
    ax.set_xlabel(r'variance (mV$^2$)', labelpad=0)
    ax.text(-0.1, 1.05, alphabet[4],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)

    barhaxes = []
    barhmax = []
    barhmin = []
    for i, (u, ls, lw, ylbl) in enumerate(zip(['x', 'y', 'z'], ['-', '-', '-'], [1, 1, 1], [r'$p_x$', r'$p_y$', r'$p_z$'])):
        ax = fig.add_subplot(gs[9+i*2:11+i*2, 4])
        if i == 0:
            ax.text(-0.1, 1.2, alphabet[9],
                horizontalalignment='center',
                verticalalignment='center',
                fontsize=16, fontweight='demibold',
                transform=ax.transAxes)
        plotting.remove_axis_junk(ax)
        barhaxes.append(ax)
        f = h5py.File(os.path.join(PSET.OUTPUTPATH,
                                   'example_parallel_network_output.h5'), 'r')
        bars = []
        p_temp = np.zeros(f['CURRENT_DIPOLE_MOMENT'].shape)
        for me_type in PSET.populationParameters['me_type']:
            bars.append((f['CURRENT_DIPOLE_MOMENT'][me_type][tind:, i]*1E-3).var()) # nA um -> 1E-3 nA m unit conversion
            p_temp += f['CURRENT_DIPOLE_MOMENT'][me_type]
        f.close()
        p_temp *= 1E-6 # nA um -> nA m unit conversion
        bars.append(p_temp[tind:, i].var())
        barhmax.append(np.array(bars).max())
        barhmin.append(np.array(bars).min())
        del p_temp
        rects = ax.barh(range(len(bars)), bars, log=True, color=colors+['k'])
        if i != 2:
            ax.set_xticklabels([])
        ax.set_yticks([])
        
        if i == 0:
            for xpos, ypos, text in zip(bars, range(len(bars)), list(PSET.populationParameters['m_type']) + ['summed']):
                ax.text(xpos, ypos, text, ha='left', va='center')        
        
        
    ax.set_xlabel(r'variance (($10^{-3}$ nA m)$^2$)', labelpad=0)
    for axh in barhaxes:
        ax.axis(ax.axis('tight'))
        axh.set_xlim(left=np.min(barhmin), right=np.max(barhmax))
    
    fig.savefig(os.path.join(PSET.OUTPUTPATH, 'figure_6.pdf'), bbox_inches='tight')



    plt.show()
